package org.jeecg.modules.ctop.controller;

import cn.com.ctop.crawler.modules.music.entity.DictMusicType;
import cn.com.ctop.crawler.modules.music.service.IDictMusicTypeService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 抖音音乐类型
 * @author jeecg-boot
 * @date   2020-10-28
 * @version V1.0
 */
@Slf4j
@Api(tags="抖音音乐类型")
@RestController
@RequestMapping("/ctop/dictMusicType")
public class DictMusicTypeController {
	@Autowired
	private IDictMusicTypeService dictMusicTypeService;

	/**
	  * 分页列表查询
	 * @param dictMusicType
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<DictMusicType>> queryPageList(DictMusicType dictMusicType,
													  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													  HttpServletRequest req) {
		Result<IPage<DictMusicType>> result = new Result<>();
		QueryWrapper<DictMusicType> queryWrapper = QueryGenerator.initQueryWrapper(dictMusicType, req.getParameterMap());
		Page<DictMusicType> page = new Page<>(pageNo, pageSize);
		IPage<DictMusicType> pageList = dictMusicTypeService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}



	/**
	  *   添加
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<DictMusicType> add(@RequestBody DictMusicType dictMusicType) {
		Result<DictMusicType> result = new Result<>();
		try {
			dictMusicTypeService.save(dictMusicType);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param dictMusicType
	 * @return
	 */
	@ApiOperation(value="抖音音乐类型-编辑", notes="抖音音乐类型-编辑")
	@PutMapping(value = "/edit")
	public Result<DictMusicType> edit(@RequestBody DictMusicType dictMusicType) {
		Result<DictMusicType> result = new Result<>();
		DictMusicType dictMusicTypeEntity = dictMusicTypeService.getById(dictMusicType.getId());
		if(dictMusicTypeEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = dictMusicTypeService.updateById(dictMusicType);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	@PostMapping("listAll")
	public JSONObject listAllType(){
		return dictMusicTypeService.listAll();
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@ApiOperation(value="抖音音乐类型-通过id删除", notes="抖音音乐类型-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			dictMusicTypeService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@ApiOperation(value="抖音音乐类型-批量删除", notes="抖音音乐类型-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<DictMusicType> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<DictMusicType> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.dictMusicTypeService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value="抖音音乐类型-通过id查询", notes="抖音音乐类型-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<DictMusicType> queryById(@RequestParam(name="id",required=true) String id) {
		Result<DictMusicType> result = new Result<>();
		DictMusicType dictMusicType = dictMusicTypeService.getById(id);
		if(dictMusicType==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(dictMusicType);
			result.setSuccess(true);
		}
		return result;
	}
}
