package org.jeecg.modules.ctop.controller;

import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.entity.JhsBakendData;
import org.jeecg.modules.ctop.service.IJhsBakendDataService;
import org.jeecg.modules.excelutil.ExcelUtils;
import org.jeecg.modules.excelutil.JhsBakendDataExcelListener;
import org.jeecg.modules.excelutil.entity.JhsBakendDataEntity;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

 /**
 * 聚划算后端数据
 * @author jeecg-boot
 * @date   2020-09-07
 * @version V1.0
 */
@Slf4j
@Api(tags="聚划算后端数据")
@RestController
@RequestMapping("/okr/jhsBakendData")
public class JhsBakendDataController {
	@Autowired
	private IJhsBakendDataService jhsBakendDataService;

	/**
	  * 分页列表查询
	 * @param jhsBakendData
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<JhsBakendData>> queryPageList(JhsBakendData jhsBakendData,
													  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													  HttpServletRequest req) {
		Result<IPage<JhsBakendData>> result = new Result<>();
		QueryWrapper<JhsBakendData> queryWrapper = QueryGenerator.initQueryWrapper(jhsBakendData, req.getParameterMap());
		Page<JhsBakendData> page = new Page<JhsBakendData>(pageNo, pageSize);
		IPage<JhsBakendData> pageList = jhsBakendDataService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param jhsBakendData
	 * @return
	 */
	@ApiOperation(value="聚划算后端数据-添加", notes="聚划算后端数据-添加")
	@PostMapping(value = "/add")
	public Result<JhsBakendData> add(@RequestBody JhsBakendData jhsBakendData) {
		Result<JhsBakendData> result = new Result<>();
		try {
			jhsBakendDataService.save(jhsBakendData);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param jhsBakendData
	 * @return
	 */
	@ApiOperation(value="聚划算后端数据-编辑", notes="聚划算后端数据-编辑")
	@PutMapping(value = "/edit")
	public Result<JhsBakendData> edit(@RequestBody JhsBakendData jhsBakendData) {
		Result<JhsBakendData> result = new Result<JhsBakendData>();
		JhsBakendData jhsBakendDataEntity = jhsBakendDataService.getById(jhsBakendData.getId());
		if(jhsBakendDataEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = jhsBakendDataService.updateById(jhsBakendData);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@ApiOperation(value="聚划算后端数据-通过id删除", notes="聚划算后端数据-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			jhsBakendDataService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@ApiOperation(value="聚划算后端数据-批量删除", notes="聚划算后端数据-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<JhsBakendData> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<JhsBakendData> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.jhsBakendDataService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value="聚划算后端数据-通过id查询", notes="聚划算后端数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<JhsBakendData> queryById(@RequestParam(name="id",required=true) String id) {
		Result<JhsBakendData> result = new Result<>();
		JhsBakendData jhsBakendData = jhsBakendDataService.getById(id);
		if(jhsBakendData==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(jhsBakendData);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<JhsBakendData> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              JhsBakendData jhsBakendData = JSON.parseObject(deString, JhsBakendData.class);
              queryWrapper = QueryGenerator.initQueryWrapper(jhsBakendData, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<JhsBakendData> pageList = jhsBakendDataService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "聚划算后端数据列表");
      mv.addObject(NormalExcelConstants.CLASS, JhsBakendData.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("聚划算后端数据列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
	  //1.读Excel
	  MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
	  Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
	  for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
		  FileInputStream fis = null;
		  try {
			  MultipartFile file = entity.getValue();
			  fis = (FileInputStream) file.getInputStream();
			  AnalysisEventListener listener = new JhsBakendDataExcelListener();
			  Boolean flag = ExcelUtils.readExcel(fis, JhsBakendDataEntity.class, listener);
			  System.out.println("导入是否成功：" + flag);
		  } catch (Exception e) {
			  e.printStackTrace();
		  } finally {
			  if (fis != null) {
				  try {
					  fis.close();
				  } catch (IOException e) {
					  e.printStackTrace();
				  }
			  }
		  }
	  }
	  return Result.ok("文件导入成功！");
  }

}
