package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.crawler.modules.account.entity.KuaishouAdAccount;
import cn.com.ctop.crawler.modules.account.service.IKuaishouAdAccountService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.entity.SysCategory;
import org.jeecg.modules.system.service.ISysCategoryService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Api(tags="快手广告主信息表")
@RestController
@RequestMapping("/ctop/kuaishou/advertiser")
public class KuaishouAdvertiserController {
    @Autowired
    private IKuaishouAdAccountService kuaishouAdAccountService;
    @Autowired
    private ISysCategoryService sysCategoryService;
    /**
     * 分页列表查询
     * @param kuaishouAdAccount
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手广告主信息表-分页列表查询")
    @ApiOperation(value="快手广告主信息表-分页列表查询", notes="快手广告主信息表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouAdAccount>> queryPageList(KuaishouAdAccount kuaishouAdAccount,
                                                          @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
                                                          @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<KuaishouAdAccount>> result = new Result<>();
        Map<String, String[]> paramMap = new HashMap<>(req.getParameterMap());
        paramMap.remove("dataProcess");
        paramMap.remove("industry");
        paramMap.remove("name");
        String industry = req.getParameter("industry");
        String name = req.getParameter("name");
        SysCategory sysCategory = sysCategoryService.getById(industry);
        kuaishouAdAccount.setDataProcess(null);
        kuaishouAdAccount.setIndustry(null);
        QueryWrapper<KuaishouAdAccount> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouAdAccount, paramMap);
        String dataProcess = req.getParameter("dataProcess");
        if (name != null) {
            queryWrapper.like("name", name);
        }
        if (sysCategory != null) {
            queryWrapper.eq("industry_category", sysCategory.getName()).or()
                    .eq("first_industry", sysCategory.getName()).or()
                    .eq("second_industry", sysCategory.getName());
        }
        if (dataProcess != null) {
            if ("industry".equals(dataProcess)) {
                queryWrapper.isNull("industry_category").or().isNull("first_industry").or().isNull("second_industry").isNull("industry_category").or().isNull("first_industry").or().isNull("second_industry").or().eq("second_industry", "").or().eq("second_industry", "").or().eq("second_industry", "");
            } else if ("company".equals(dataProcess)) {
                queryWrapper.isNull("company").or().eq("company", "");
            } else if ("product".equals(dataProcess)) {
                queryWrapper.isNull("product").or().eq("product", "");
            } else if ("full".equals(dataProcess)) {
                queryWrapper.isNotNull("industry_category");
                queryWrapper.ne("industry_category", "");
                queryWrapper.isNotNull("first_industry");
                queryWrapper.ne("first_industry", "");
                queryWrapper.isNotNull("second_industry");
                queryWrapper.ne("second_industry", "");
                queryWrapper.isNotNull("company");
                queryWrapper.ne("company", "");
                queryWrapper.isNotNull("product");
                queryWrapper.ne("product", "");
            }
        }

        Page<KuaishouAdAccount> page = new Page<>(pageNo, pageSize);
        IPage<KuaishouAdAccount> pageList = kuaishouAdAccountService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     *   添加
     * @param kuaishouAdAccount
     * @return
     */
    @AutoLog(value = "快手广告主信息表-添加")
    @ApiOperation(value="快手广告主信息表-添加", notes="快手广告主信息表-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouAdAccount> add(@RequestBody KuaishouAdAccount kuaishouAdAccount) {
        Result<KuaishouAdAccount> result = new Result<>();
        try {
            String industry = kuaishouAdAccount.getIndustry();
            SysCategory secondCategory = sysCategoryService.getById(industry);
            kuaishouAdAccount.setSecondIndustry(secondCategory.getName());
            SysCategory firstCategory = sysCategoryService.getById(secondCategory.getPid());
            kuaishouAdAccount.setFirstIndustry(firstCategory.getName());
            SysCategory industryCategory = sysCategoryService.getById(firstCategory.getPid());
            kuaishouAdAccount.setIndustryCategory(industryCategory.getName());
            kuaishouAdAccountService.save(kuaishouAdAccount);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(),e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     *  编辑
     * @param kuaishouAdAccount
     * @return
     */
    @AutoLog(value = "快手广告主信息表-编辑")
    @ApiOperation(value="快手广告主信息表-编辑", notes="快手广告主信息表-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouAdAccount> edit(@RequestBody KuaishouAdAccount kuaishouAdAccount) {
        Result<KuaishouAdAccount> result = new Result<>();
        KuaishouAdAccount kuaishouAdAccountEntity = kuaishouAdAccountService.getById(kuaishouAdAccount.getKsid());

        if(kuaishouAdAccountEntity==null) {
            result.error500("未找到对应实体");
        }else {
            String industry = kuaishouAdAccount.getIndustry();
            SysCategory secondCategory = sysCategoryService.getById(industry);
            kuaishouAdAccount.setSecondIndustry(secondCategory.getName());
            SysCategory firstCategory = sysCategoryService.getById(secondCategory.getPid());
            kuaishouAdAccount.setFirstIndustry(firstCategory.getName());
            SysCategory industryCategory = sysCategoryService.getById(firstCategory.getPid());
            kuaishouAdAccount.setIndustryCategory(industryCategory.getName());
            QueryWrapper<KuaishouAdAccount> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("ksid", kuaishouAdAccount.getKsid());
            boolean ok = kuaishouAdAccountService.update(kuaishouAdAccount, queryWrapper);
            if(ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     *   通过id删除
     * @param id
     * @return
     */
    @AutoLog(value = "快手广告主信息表-通过id删除")
    @ApiOperation(value="快手广告主信息表-通过id删除", notes="快手广告主信息表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouAdAccountService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败",e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     *  批量删除
     * @param ids
     * @return
     */
    @AutoLog(value = "快手广告主信息表-批量删除")
    @ApiOperation(value="快手广告主信息表-批量删除", notes="快手广告主信息表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouAdAccount> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
        Result<KuaishouAdAccount> result = new Result<>();
        if(ids==null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        }else {
            this.kuaishouAdAccountService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     * @param id
     * @return
     */
    @AutoLog(value = "快手广告主信息表-通过id查询")
    @ApiOperation(value="快手广告主信息表-通过id查询", notes="快手广告主信息表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouAdAccount> queryById(@RequestParam(name="id",required=true) String id) {
        Result<KuaishouAdAccount> result = new Result<>();
        KuaishouAdAccount kuaishouAdAccount = kuaishouAdAccountService.getById(id);
        if(kuaishouAdAccount==null) {
            result.error500("未找到对应实体");
        }else {
            result.setResult(kuaishouAdAccount);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouAdAccount> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouAdAccount kuaishouAdAccount = JSON.parseObject(deString, KuaishouAdAccount.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouAdAccount, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouAdAccount> pageList = kuaishouAdAccountService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手广告主信息表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouAdAccount.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手广告主信息表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @PostMapping("/importExcel")
    public Result<Object> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouAdAccount> listKuaishouAdAccounts = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouAdAccount.class, params);
                kuaishouAdAccountService.saveBatch(listKuaishouAdAccounts);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouAdAccounts.size());
            } catch (Exception e) {
                log.error(e.getMessage(),e);
                return Result.error("文件导入失败:"+e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }
    @ResponseBody
    @GetMapping("/companysugest")
    public Result<List<String>> getCompanySugest(@RequestParam("company") String company){
        Result<List<String>> result = new Result<>();
        List<String> list = kuaishouAdAccountService.companySugest(company);
        result.setResult(list);
        result.setSuccess(true);
        return result;
    }

    @GetMapping(value = "/productsugest")
    public Result<List<String>> getProductSugest(@RequestParam("product") String product){
        Result<List<String>> result = new Result<>();
        List<String> list = kuaishouAdAccountService.productSugest(product);
        result.setResult(list);
        result.setSuccess(true);
        return result;
    }
}
