package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.crawler.modules.account.entity.KuaishouAppAccount;
import cn.com.ctop.crawler.modules.account.service.IKuaishouAppAccountService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

 /**
  * 快手APP账号表
  * @author jeecg-boot
  * @date 2019-06-28
  * @version V1.0
 */
@Slf4j
@Api(tags="快手APP账号表")
@RestController
@RequestMapping("/ctop/kuaishouAppAccount")
public class KuaishouAppAccountController {
	@Autowired
	private IKuaishouAppAccountService kuaishouAppAccountService;

     /**
	  * 分页列表查询
	 * @param kuaishouAppAccount
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "快手APP账号表-分页列表查询")
	@ApiOperation(value="快手APP账号表-分页列表查询", notes="快手APP账号表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<KuaishouAppAccount>> queryPageList(KuaishouAppAccount kuaishouAppAccount,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
        Result<IPage<KuaishouAppAccount>> result = new Result<>();
		QueryWrapper<KuaishouAppAccount> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouAppAccount, req.getParameterMap());
        Page<KuaishouAppAccount> page = new Page<>(pageNo, pageSize);
		IPage<KuaishouAppAccount> pageList = kuaishouAppAccountService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

     /**
	  *   添加
	 * @param kuaishouAppAccount
	 * @return
	 */
	@AutoLog(value = "快手APP账号表-添加")
	@ApiOperation(value="快手APP账号表-添加", notes="快手APP账号表-添加")
	@PostMapping(value = "/add")
	public Result<KuaishouAppAccount> add(@RequestBody KuaishouAppAccount kuaishouAppAccount) {
        Result<KuaishouAppAccount> result = new Result<>();
		try {
			kuaishouAppAccountService.save(kuaishouAppAccount);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

     /**
	  *  编辑
	 * @param kuaishouAppAccount
	 * @return
	 */
	@AutoLog(value = "快手APP账号表-编辑")
	@ApiOperation(value="快手APP账号表-编辑", notes="快手APP账号表-编辑")
	@PutMapping(value = "/edit")
	public Result<KuaishouAppAccount> edit(@RequestBody KuaishouAppAccount kuaishouAppAccount) {
        Result<KuaishouAppAccount> result = new Result<>();
		KuaishouAppAccount kuaishouAppAccountEntity = kuaishouAppAccountService.getById(kuaishouAppAccount.getId());
		if(kuaishouAppAccountEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = kuaishouAppAccountService.updateById(kuaishouAppAccount);
			if(ok) {
				result.success("修改成功!");
			}
		}

        return result;
	}

     /**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "快手APP账号表-通过id删除")
	@ApiOperation(value="快手APP账号表-通过id删除", notes="快手APP账号表-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<KuaishouAppAccount> delete(@RequestParam(name="id",required=true) String id) {
        Result<KuaishouAppAccount> result = new Result<>();
		KuaishouAppAccount kuaishouAppAccount = kuaishouAppAccountService.getById(id);
		if(kuaishouAppAccount==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = kuaishouAppAccountService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}

        return result;
	}

     /**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "快手APP账号表-批量删除")
	@ApiOperation(value="快手APP账号表-批量删除", notes="快手APP账号表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<KuaishouAppAccount> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<KuaishouAppAccount> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.kuaishouAppAccountService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
			result.success("删除成功!");
		}
		return result;
	}

     /**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "快手APP账号表-通过id查询")
	@ApiOperation(value="快手APP账号表-通过id查询", notes="快手APP账号表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<KuaishouAppAccount> queryById(@RequestParam(name="id",required=true) String id) {
        Result<KuaishouAppAccount> result = new Result<>();
		KuaishouAppAccount kuaishouAppAccount = kuaishouAppAccountService.getById(id);
		if(kuaishouAppAccount==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(kuaishouAppAccount);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<KuaishouAppAccount> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              KuaishouAppAccount kuaishouAppAccount = JSON.parseObject(deString, KuaishouAppAccount.class);
              queryWrapper = QueryGenerator.initQueryWrapper(kuaishouAppAccount, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<KuaishouAppAccount> pageList = kuaishouAppAccountService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "快手APP账号表列表");
      mv.addObject(NormalExcelConstants.CLASS, KuaishouAppAccount.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手APP账号表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @PostMapping("/importExcel")
  public Result<Object> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
		  MultipartFile file = entity.getValue();
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<KuaishouAppAccount> listKuaishouAppAccounts = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouAppAccount.class, params);
              for (KuaishouAppAccount kuaishouAppAccountExcel : listKuaishouAppAccounts) {
                  kuaishouAppAccountService.save(kuaishouAppAccountExcel);
              }
              return Result.ok("文件导入成功！数据行数:" + listKuaishouAppAccounts.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
