package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.ctop.entity.KuaishouEffectVideoInfo;
import org.jeecg.modules.ctop.service.IKuaishouEffectVideoInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 快手视频数据清洗信息
 * @author jeecg-boot
 * @date   2020-07-23
 * @version V1.0
 */
@Slf4j
@Api(tags="快手视频数据清洗信息")
@RestController
@RequestMapping("/ctop/kuaishouEffectVideoInfo")
public class KuaishouEffectVideoInfoController {
	@Autowired
	private IKuaishouEffectVideoInfoService kuaishouEffectVideoInfoService;

	/**
	  * 分页列表查询
	 * @param kuaishouEffectVideoInfo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "快手视频数据清洗信息-分页列表查询")
	@ApiOperation(value="快手视频数据清洗信息-分页列表查询", notes="快手视频数据清洗信息-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<KuaishouEffectVideoInfo>> queryPageList(KuaishouEffectVideoInfo kuaishouEffectVideoInfo,
																@RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
																@RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
																HttpServletRequest req) {
		Result<IPage<KuaishouEffectVideoInfo>> result = new Result<>();
		QueryWrapper<KuaishouEffectVideoInfo> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouEffectVideoInfo, req.getParameterMap());
		Page<KuaishouEffectVideoInfo> page = new Page<>(pageNo, pageSize);
		IPage<KuaishouEffectVideoInfo> pageList = kuaishouEffectVideoInfoService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param kuaishouEffectVideoInfo
	 * @return
	 */
	@AutoLog(value = "快手视频数据清洗信息-添加")
	@ApiOperation(value="快手视频数据清洗信息-添加", notes="快手视频数据清洗信息-添加")
	@PostMapping(value = "/add")
	public Result<KuaishouEffectVideoInfo> add(@RequestBody KuaishouEffectVideoInfo kuaishouEffectVideoInfo) {
		Result<KuaishouEffectVideoInfo> result = new Result<>();
		try {
			kuaishouEffectVideoInfoService.save(kuaishouEffectVideoInfo);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param kuaishouEffectVideoInfo
	 * @return
	 */
	@AutoLog(value = "快手视频数据清洗信息-编辑")
	@ApiOperation(value="快手视频数据清洗信息-编辑", notes="快手视频数据清洗信息-编辑")
	@PutMapping(value = "/edit")
	public Result<KuaishouEffectVideoInfo> edit(@RequestBody KuaishouEffectVideoInfo kuaishouEffectVideoInfo) {
		Result<KuaishouEffectVideoInfo> result = new Result<>();
		KuaishouEffectVideoInfo kuaishouEffectVideoInfoEntity = kuaishouEffectVideoInfoService.getById(kuaishouEffectVideoInfo.getId());
		if(kuaishouEffectVideoInfoEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = kuaishouEffectVideoInfoService.updateById(kuaishouEffectVideoInfo);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "快手视频数据清洗信息-通过id删除")
	@ApiOperation(value="快手视频数据清洗信息-通过id删除", notes="快手视频数据清洗信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Object> delete(@RequestParam(name="id",required=true) String id) {
		try {
			kuaishouEffectVideoInfoService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "快手视频数据清洗信息-批量删除")
	@ApiOperation(value="快手视频数据清洗信息-批量删除", notes="快手视频数据清洗信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<KuaishouEffectVideoInfo> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<KuaishouEffectVideoInfo> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.kuaishouEffectVideoInfoService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "快手视频数据清洗信息-通过id查询")
	@ApiOperation(value="快手视频数据清洗信息-通过id查询", notes="快手视频数据清洗信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<KuaishouEffectVideoInfo> queryById(@RequestParam(name="id",required=true) String id) {
		Result<KuaishouEffectVideoInfo> result = new Result<>();
		KuaishouEffectVideoInfo kuaishouEffectVideoInfo = kuaishouEffectVideoInfoService.getById(id);
		if(kuaishouEffectVideoInfo==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(kuaishouEffectVideoInfo);
			result.setSuccess(true);
		}
		return result;
	}
}
