package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.MaterialAscription;
import cn.com.ctop.common.module.service.IMaterialAscriptionService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 素材归属标
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-11-23
 */
@Slf4j
@Api(tags = "素材归属")
@RestController
@RequestMapping("/ctop/materialAscription")
public class MaterialAscriptionController {
    @Autowired
    private IMaterialAscriptionService materialAscriptionService;

    /**
     * 分页列表查询
     *
     * @param materialAscription
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "素材归属标-分页列表查询")
    @ApiOperation(value = "素材归属标-分页列表查询", notes = "素材归属标-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<MaterialAscription>> queryPageList(MaterialAscription materialAscription,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<MaterialAscription>> result = new Result<>();
        QueryWrapper<MaterialAscription> queryWrapper = QueryGenerator.initQueryWrapper(materialAscription, req.getParameterMap());
        Page<MaterialAscription> page = new Page<>(pageNo, pageSize);
        IPage<MaterialAscription> pageList = materialAscriptionService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    @GetMapping(value = "/getDetail")
    public Result<JSONObject> getDetail(String code) {
        Result<JSONObject> result = new Result<>();
        JSONObject json = materialAscriptionService.getDetailByCode(code);
        result.setSuccess(true);
        result.setResult(json);
        return result;
    }

    /**
     * 添加
     *
     * @param materialAscription
     * @return
     */
    @AutoLog(value = "素材归属标-添加")
    @ApiOperation(value = "素材归属标-添加", notes = "素材归属标-添加")
    @PostMapping(value = "/add")
    public Result<MaterialAscription> add(@RequestBody MaterialAscription materialAscription) {
        Result<MaterialAscription> result = new Result<>();
        try {
            materialAscriptionService.save(materialAscription);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param materialAscription
     * @return
     */
    @AutoLog(value = "素材归属标-编辑")
    @ApiOperation(value = "素材归属标-编辑", notes = "素材归属标-编辑")
    @PutMapping(value = "/edit")
    public Result<MaterialAscription> edit(@RequestBody MaterialAscription materialAscription) {
        Result<MaterialAscription> result = new Result<>();
        MaterialAscription materialAscriptionEntity = materialAscriptionService.getById(materialAscription.getId());
        if (materialAscriptionEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = materialAscriptionService.updateById(materialAscription);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    @PutMapping(value = "/update")
    public Result<MaterialAscription> update(@RequestBody MaterialAscription materialAscription) {
        Result<MaterialAscription> result = new Result<>();
        QueryWrapper<MaterialAscription> ascriptionQueryWrapper = new QueryWrapper<>();
        ascriptionQueryWrapper.eq("material_id", materialAscription.getMaterialId());
        boolean ok = materialAscriptionService.update(materialAscription, ascriptionQueryWrapper);
        if (ok) {
            result.success("修改成功!");
        }
        return result;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材归属标-通过id删除")
    @ApiOperation(value = "素材归属标-通过id删除", notes = "素材归属标-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            materialAscriptionService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "素材归属标-批量删除")
    @ApiOperation(value = "素材归属标-批量删除", notes = "素材归属标-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<MaterialAscription> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<MaterialAscription> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.materialAscriptionService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材归属标-通过id查询")
    @ApiOperation(value = "素材归属标-通过id查询", notes = "素材归属标-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<MaterialAscription> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<MaterialAscription> result = new Result<>();
        MaterialAscription materialAscription = materialAscriptionService.getById(id);
        if (materialAscription == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(materialAscription);
            result.setSuccess(true);
        }
        return result;
    }
}
