package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.MaterialTagInfo;
import cn.com.ctop.common.module.service.IMaterialTagInfoService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Map;

/**
 * 素材标签信息
 * @author jeecg-boot
 * @date   2020-08-27
 * @version V1.0
 */
@Slf4j
@Api(tags="素材标签信息")
@RestController
@RequestMapping("/ctop/materialTagInfo")
public class MaterialTagInfoController {
	@Autowired
	private IMaterialTagInfoService materialTagInfoService;

	/**
	  * 分页列表查询
	 * @param materialTagInfo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<MaterialTagInfo>> queryPageList(MaterialTagInfo materialTagInfo,
                                                        @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
                                                        @RequestParam(name="pageSize", defaultValue="100") Integer pageSize,
                                                        HttpServletRequest req) {
		Result<IPage<MaterialTagInfo>> result = new Result<>();
		QueryWrapper<MaterialTagInfo> queryWrapper = QueryGenerator.initQueryWrapper(materialTagInfo, req.getParameterMap());
		Page<MaterialTagInfo> page = new Page<>(pageNo, pageSize);
		IPage<MaterialTagInfo> pageList = materialTagInfoService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	@GetMapping(value = "/detail")
	public Map<String,Object> detailList(MaterialTagInfo materialTagInfo) {
		return materialTagInfoService.getTagDetail(materialTagInfo);
	}

	/**
	  *   添加
	 * @param materialTagInfo
	 * @return
	 */
	@ApiOperation(value="素材标签信息-添加", notes="素材标签信息-添加")
	@PostMapping(value = "/add")
	public Result<MaterialTagInfo> add(@RequestBody MaterialTagInfo materialTagInfo) {
		Result<MaterialTagInfo> result = new Result<>();
		try {
			materialTagInfoService.save(materialTagInfo);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param materialTagInfo
	 * @return
	 */
	@ApiOperation(value="素材标签信息-编辑", notes="素材标签信息-编辑")
	@PutMapping(value = "/edit")
	public Result<MaterialTagInfo> edit(@RequestBody MaterialTagInfo materialTagInfo) {
		Result<MaterialTagInfo> result = new Result<>();
		MaterialTagInfo materialTagInfoEntity = materialTagInfoService.getById(materialTagInfo.getId());
		if(materialTagInfoEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = materialTagInfoService.updateById(materialTagInfo);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@ApiOperation(value="素材标签信息-通过id删除", notes="素材标签信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			materialTagInfoService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@ApiOperation(value="素材标签信息-批量删除", notes="素材标签信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<MaterialTagInfo> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<MaterialTagInfo> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			materialTagInfoService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	@PostMapping("mark")
	public Map<String,Object> mark(@RequestBody JSONObject data){
		LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		return materialTagInfoService.mark(data,user);
	}
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value="素材标签信息-通过id查询", notes="素材标签信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<MaterialTagInfo> queryById(@RequestParam(name="id",required=true) String id) {
		Result<MaterialTagInfo> result = new Result<>();
		MaterialTagInfo materialTagInfo = materialTagInfoService.getById(id);
		if(materialTagInfo==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(materialTagInfo);
			result.setSuccess(true);
		}
		return result;
	}

}
