package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.SupplierWatermark;
import cn.com.ctop.common.module.service.ISupplierWatermarkService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 供应商水印
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-07-28
 */
@Slf4j
@Api(tags = "供应商水印")
@RestController
@RequestMapping("/ctop/supplierWatermark")
public class SupplierWatermarkController {
    @Autowired
    private ISupplierWatermarkService supplierWatermarkService;

    /**
     * 分页列表查询
     *
     * @param supplierWatermark
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "供应商水印-分页列表查询")
    @ApiOperation(value = "供应商水印-分页列表查询", notes = "供应商水印-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<SupplierWatermark>> queryPageList(SupplierWatermark supplierWatermark,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<SupplierWatermark>> result = new Result<>();
        QueryWrapper<SupplierWatermark> queryWrapper = QueryGenerator.initQueryWrapper(supplierWatermark, req.getParameterMap());
        Page<SupplierWatermark> page = new Page<SupplierWatermark>(pageNo, pageSize);
        IPage<SupplierWatermark> pageList = supplierWatermarkService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param supplierWatermark
     * @return
     */
    @AutoLog(value = "供应商水印-添加")
    @ApiOperation(value = "供应商水印-添加", notes = "供应商水印-添加")
    @PostMapping(value = "/add")
    public Result<SupplierWatermark> add(@RequestBody SupplierWatermark supplierWatermark) {
        Result<SupplierWatermark> result = new Result<>();
        try {
            supplierWatermarkService.save(supplierWatermark);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param supplierWatermark
     * @return
     */
    @AutoLog(value = "供应商水印-编辑")
    @ApiOperation(value = "供应商水印-编辑", notes = "供应商水印-编辑")
    @PutMapping(value = "/edit")
    public Result<SupplierWatermark> edit(@RequestBody SupplierWatermark supplierWatermark) {
        Result<SupplierWatermark> result = new Result<SupplierWatermark>();
        SupplierWatermark supplierWatermarkEntity = supplierWatermarkService.getById(supplierWatermark.getId());
        if (supplierWatermarkEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = supplierWatermarkService.updateById(supplierWatermark);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "供应商水印-通过id删除")
    @ApiOperation(value = "供应商水印-通过id删除", notes = "供应商水印-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            supplierWatermarkService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "供应商水印-批量删除")
    @ApiOperation(value = "供应商水印-批量删除", notes = "供应商水印-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<SupplierWatermark> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<SupplierWatermark> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.supplierWatermarkService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "供应商水印-通过id查询")
    @ApiOperation(value = "供应商水印-通过id查询", notes = "供应商水印-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<SupplierWatermark> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<SupplierWatermark> result = new Result<>();
        SupplierWatermark supplierWatermark = supplierWatermarkService.getById(id);
        if (supplierWatermark == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(supplierWatermark);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<SupplierWatermark> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                SupplierWatermark supplierWatermark = JSON.parseObject(deString, SupplierWatermark.class);
                queryWrapper = QueryGenerator.initQueryWrapper(supplierWatermark, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<SupplierWatermark> pageList = supplierWatermarkService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "供应商水印列表");
        mv.addObject(NormalExcelConstants.CLASS, SupplierWatermark.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("供应商水印列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<SupplierWatermark> listSupplierWatermarks = ExcelImportUtil.importExcel(file.getInputStream(), SupplierWatermark.class, params);
                supplierWatermarkService.saveBatch(listSupplierWatermarks);
                return Result.ok("文件导入成功！数据行数:" + listSupplierWatermarks.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
