package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import cn.com.ctop.toutiao.modules.report.service.IBytedancePlanDailyReportService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.UReportFile;
import org.jeecg.modules.ctop.entity.UReportFileBindUser;
import org.jeecg.modules.ctop.service.IUReportFileBindUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 用户绑定报表信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-01-16
 */
@Slf4j
@Api(tags = "用户绑定报表信息")
@RestController
@RequestMapping("/ctop/uReportFileBindUser")
public class UReportFileBindUserController {
    @Autowired
    private IUReportFileBindUserService uReportFileBindUserService;
    @Autowired
    private IBytedancePlanDailyReportService planDailyReportService;
    @PostMapping("loadRegistOrderDate")
    public Map<String,Object>loadData(String data,String date){
        Map<String,Object>result = new HashMap<>();
        try {
            planDailyReportService.loadData(data,date);
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
            return result;
        }catch (Exception e){
            e.printStackTrace();
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_SERVER_ERROR);
            return result;
        }
    }
    /**
     * 分页列表查询
     *
     * @param uReportFileBindUser
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "用户绑定报表信息-分页列表查询")
    @ApiOperation(value = "用户绑定报表信息-分页列表查询", notes = "用户绑定报表信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UReportFileBindUser>> queryPageList(UReportFileBindUser uReportFileBindUser,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<UReportFileBindUser>> result = new Result<>();
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        uReportFileBindUser.setUserId(user.getId());
        String fileName = null;
        if (null != uReportFileBindUser.getFileName() && !"".equals(uReportFileBindUser.getFileName().trim())) {
            fileName = uReportFileBindUser.getFileName();
            uReportFileBindUser.setFileName(null);
        }
        QueryWrapper<UReportFileBindUser> queryWrapper = QueryGenerator.initQueryWrapper(uReportFileBindUser, req.getParameterMap());
        if (null != fileName) {
            queryWrapper.like("file_name", fileName);
        }
        Page<UReportFileBindUser> page = new Page<>(pageNo, pageSize);
        IPage<UReportFileBindUser> pageList = uReportFileBindUserService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 分页列表查询
     *
     * @param req
     * @return
     */
    @AutoLog(value = "用户绑定报表信息-分页列表查询")
    @ApiOperation(value = "用户绑定报表信息-分页列表查询", notes = "用户绑定报表信息-分页列表查询")
    @PostMapping(value = "/userList")
    public Map<String, Object> queryPageList(@RequestBody JSONObject data,
                                             HttpServletRequest req) {
        return uReportFileBindUserService.getListByFileId(data);
    }

    /**
     * 添加
     *
     * @param uReportFileBindUser
     * @return
     */
    @AutoLog(value = "用户绑定报表信息-添加")
    @ApiOperation(value = "用户绑定报表信息-添加", notes = "用户绑定报表信息-添加")
    @PostMapping(value = "/add")
    public Result<UReportFileBindUser> add(@RequestBody UReportFileBindUser uReportFileBindUser) {
        Result<UReportFileBindUser> result = new Result<>();
        try {
            uReportFileBindUserService.save(uReportFileBindUser);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param uReportFileBindUser
     * @return
     */
    @AutoLog(value = "用户绑定报表信息-编辑")
    @ApiOperation(value = "用户绑定报表信息-编辑", notes = "用户绑定报表信息-编辑")
    @PutMapping(value = "/edit")
    public Result<UReportFileBindUser> edit(@RequestBody UReportFileBindUser uReportFileBindUser) {
        Result<UReportFileBindUser> result = new Result<>();
        UReportFileBindUser uReportFileBindUserEntity = uReportFileBindUserService.getById(uReportFileBindUser.getId());
        if (uReportFileBindUserEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = uReportFileBindUserService.updateById(uReportFileBindUser);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户绑定报表信息-通过id删除")
    @ApiOperation(value = "用户绑定报表信息-通过id删除", notes = "用户绑定报表信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            uReportFileBindUserService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "用户绑定报表信息-批量删除")
    @ApiOperation(value = "用户绑定报表信息-批量删除", notes = "用户绑定报表信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<UReportFileBindUser> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<UReportFileBindUser> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.uReportFileBindUserService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户绑定报表信息-通过id查询")
    @ApiOperation(value = "用户绑定报表信息-通过id查询", notes = "用户绑定报表信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UReportFileBindUser> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<UReportFileBindUser> result = new Result<>();
        UReportFileBindUser uReportFileBindUser = uReportFileBindUserService.getById(id);
        if (uReportFileBindUser == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(uReportFileBindUser);
            result.setSuccess(true);
        }
        return result;
    }


    /**
     *  订阅按钮
     */
    @PostMapping(value = "/subscribe")
    public Result subscribe(@RequestParam(name = "fileId") String fileId) {
        Result<UReportFile> result = new Result<>();
        QueryWrapper<UReportFileBindUser> queryWrapper=new QueryWrapper();
        queryWrapper.eq("file_id",fileId);
        queryWrapper.ne("user_id","e9ca23d68d884d4ebb19d07889727dae");
        List<UReportFileBindUser> uReportFileBindUser = uReportFileBindUserService.list(queryWrapper);
        if (uReportFileBindUser.isEmpty()) {
            result.error500("请绑定用户后订阅");
        } else {
            uReportFileBindUser.forEach(it->it.setIsSubscribe(true));
            boolean ok = uReportFileBindUserService.updateBatchById(uReportFileBindUser);
            if (ok) {
                result.success("订阅成功!");
            }
        }
        return result;
    }

}
