package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.VideoPart;
import cn.com.ctop.common.module.service.IVideoPartService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 视频片段
 * @author jeecg-boot
 * @date   2019-12-16
 * @version V1.0
 */
@Slf4j
@Api(tags="视频片段")
@RestController
@RequestMapping("/ctop/videoPart")
public class VideoPartController {
	@Autowired
	private IVideoPartService videoPartService;

	/**
	  * 分页列表查询
	 * @param videoPart
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "视频片段-分页列表查询")
	@ApiOperation(value="视频片段-分页列表查询", notes="视频片段-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<VideoPart>> queryPageList(VideoPart videoPart,
												  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
												  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
												  HttpServletRequest req) {
        Result<IPage<VideoPart>> result = new Result<>();
		QueryWrapper<VideoPart> queryWrapper = QueryGenerator.initQueryWrapper(videoPart, req.getParameterMap());
        Page<VideoPart> page = new Page<>(pageNo, pageSize);
		IPage<VideoPart> pageList = videoPartService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param videoPart
	 * @return
	 */
	@AutoLog(value = "视频片段-添加")
	@ApiOperation(value="视频片段-添加", notes="视频片段-添加")
	@PostMapping(value = "/add")
	public Result<VideoPart> add(@RequestBody VideoPart videoPart) {
        Result<VideoPart> result = new Result<>();
		try {
			videoPartService.save(videoPart);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param videoPart
	 * @return
	 */
	@AutoLog(value = "视频片段-编辑")
	@ApiOperation(value="视频片段-编辑", notes="视频片段-编辑")
	@PutMapping(value = "/edit")
	public Result<VideoPart> edit(@RequestBody VideoPart videoPart) {
        Result<VideoPart> result = new Result<>();
		VideoPart videoPartEntity = videoPartService.getById(videoPart.getId());
		if(videoPartEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = videoPartService.updateById(videoPart);
			if(ok) {
				result.success("修改成功!");
			}
		}

        return result;
	}

     /**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "视频片段-通过id删除")
	@ApiOperation(value="视频片段-通过id删除", notes="视频片段-通过id删除")
	@DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
		try {
			videoPartService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

     /**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "视频片段-批量删除")
	@ApiOperation(value="视频片段-批量删除", notes="视频片段-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<VideoPart> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
        Result<VideoPart> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.videoPartService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

     /**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "视频片段-通过id查询")
	@ApiOperation(value="视频片段-通过id查询", notes="视频片段-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<VideoPart> queryById(@RequestParam(name="id",required=true) String id) {
        Result<VideoPart> result = new Result<>();
		VideoPart videoPart = videoPartService.getById(id);
		if(videoPart==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(videoPart);
			result.setSuccess(true);
		}
		return result;
	}
}
