package org.jeecg.modules.opencv.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.opencv.entity.ModelVideoCompositeInfo;
import org.jeecg.modules.opencv.service.IModelVideoCompositeInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 模板合成视频
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-17
 */
@Slf4j
@Api(tags = "模板合成视频")
@RestController
@RequestMapping("/ctop/modelVideoCompositeInfo")
public class ModelVideoCompositeInfoController {
    @Autowired
    private IModelVideoCompositeInfoService modelVideoCompositeInfoService;

    /**
     * 分页列表查询
     *
     * @param modelVideoCompositeInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "模板合成视频-分页列表查询")
    @ApiOperation(value = "模板合成视频-分页列表查询", notes = "模板合成视频-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ModelVideoCompositeInfo>> queryPageList(ModelVideoCompositeInfo modelVideoCompositeInfo,
                                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                HttpServletRequest req) {
        Result<IPage<ModelVideoCompositeInfo>> result = new Result<>();
        QueryWrapper<ModelVideoCompositeInfo> queryWrapper = QueryGenerator.initQueryWrapper(modelVideoCompositeInfo, req.getParameterMap());
        Page<ModelVideoCompositeInfo> page = new Page<>(pageNo, pageSize);
        IPage<ModelVideoCompositeInfo> pageList = modelVideoCompositeInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param modelVideoCompositeInfo
     * @return
     */
    @AutoLog(value = "模板合成视频-添加")
    @ApiOperation(value = "模板合成视频-添加", notes = "模板合成视频-添加")
    @PostMapping(value = "/add")
    public Result<ModelVideoCompositeInfo> add(@RequestBody ModelVideoCompositeInfo modelVideoCompositeInfo) {
        Result<ModelVideoCompositeInfo> result = new Result<ModelVideoCompositeInfo>();
        try {
            modelVideoCompositeInfoService.save(modelVideoCompositeInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param modelVideoCompositeInfo
     * @return
     */
    @AutoLog(value = "模板合成视频-编辑")
    @ApiOperation(value = "模板合成视频-编辑", notes = "模板合成视频-编辑")
    @PutMapping(value = "/edit")
    public Result<ModelVideoCompositeInfo> edit(@RequestBody ModelVideoCompositeInfo modelVideoCompositeInfo) {
        Result<ModelVideoCompositeInfo> result = new Result<ModelVideoCompositeInfo>();
        ModelVideoCompositeInfo modelVideoCompositeInfoEntity = modelVideoCompositeInfoService.getById(modelVideoCompositeInfo.getId());
        if (modelVideoCompositeInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = modelVideoCompositeInfoService.updateById(modelVideoCompositeInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "模板合成视频-通过id删除")
    @ApiOperation(value = "模板合成视频-通过id删除", notes = "模板合成视频-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            modelVideoCompositeInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "模板合成视频-批量删除")
    @ApiOperation(value = "模板合成视频-批量删除", notes = "模板合成视频-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ModelVideoCompositeInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ModelVideoCompositeInfo> result = new Result<ModelVideoCompositeInfo>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.modelVideoCompositeInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "模板合成视频-通过id查询")
    @ApiOperation(value = "模板合成视频-通过id查询", notes = "模板合成视频-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ModelVideoCompositeInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ModelVideoCompositeInfo> result = new Result<ModelVideoCompositeInfo>();
        ModelVideoCompositeInfo modelVideoCompositeInfo = modelVideoCompositeInfoService.getById(id);
        if (modelVideoCompositeInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(modelVideoCompositeInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
