package org.jeecg.modules.opencv.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.opencv.entity.ModelVideoInfo;
import org.jeecg.modules.opencv.service.IModelVideoInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 模板视频信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-17
 */
@Slf4j
@Api(tags = "模板视频信息")
@RestController
@RequestMapping("/ctop/modelVideoInfo")
public class ModelVideoInfoController {
    @Autowired
    private IModelVideoInfoService modelVideoInfoService;

    /**
     * 分页列表查询
     *
     * @param modelVideoInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "模板视频信息-分页列表查询")
    @ApiOperation(value = "模板视频信息-分页列表查询", notes = "模板视频信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ModelVideoInfo>> queryPageList(ModelVideoInfo modelVideoInfo,
                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                       HttpServletRequest req) {
        Result<IPage<ModelVideoInfo>> result = new Result<>();
        String title = modelVideoInfo.getTitle();
        modelVideoInfo.setTitle(null);
        Integer orderType = modelVideoInfo.getOrderType();
        modelVideoInfo.setOrderType(null);
        QueryWrapper<ModelVideoInfo> queryWrapper = QueryGenerator.initQueryWrapper(modelVideoInfo, req.getParameterMap());
        if (null != orderType && orderType == 1) {
            queryWrapper.orderByDesc("model_type", "id");
        } else if (orderType == 2) {
            queryWrapper.orderByDesc("id");
        }
        if (null != title && !"".equals(title.trim())) {
            queryWrapper.like("title", title);
        }
        Page<ModelVideoInfo> page = new Page<>(pageNo, pageSize);
        IPage<ModelVideoInfo> pageList = modelVideoInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    /**
     * 添加
     *
     * @param modelVideoInfo
     * @return
     */
    @AutoLog(value = "模板视频信息-添加")
    @ApiOperation(value = "模板视频信息-添加", notes = "模板视频信息-添加")
    @PostMapping(value = "/add")
    public Result<ModelVideoInfo> add(@RequestBody ModelVideoInfo modelVideoInfo) {
        Result<ModelVideoInfo> result = new Result<>();
        try {
            modelVideoInfo.setModelVideoUrl("https:" + modelVideoInfo.getModelVideoUrl());
            modelVideoInfo.setStatus(1);
            modelVideoInfo.setModelType(1);
            modelVideoInfoService.save(modelVideoInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param modelVideoInfo
     * @return
     */
    @AutoLog(value = "模板视频信息-编辑")
    @ApiOperation(value = "模板视频信息-编辑", notes = "模板视频信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ModelVideoInfo> edit(@RequestBody ModelVideoInfo modelVideoInfo) {
        Result<ModelVideoInfo> result = new Result<>();
        ModelVideoInfo modelVideoInfoEntity = modelVideoInfoService.getById(modelVideoInfo.getId());
        if (modelVideoInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = modelVideoInfoService.updateById(modelVideoInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id id
     * @return 操作结果
     */
    @AutoLog(value = "模板视频信息-通过id删除")
    @ApiOperation(value = "模板视频信息-通过id删除", notes = "模板视频信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            modelVideoInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids id信息
     * @return 操作结果
     */
    @AutoLog(value = "模板视频信息-批量删除")
    @ApiOperation(value = "模板视频信息-批量删除", notes = "模板视频信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ModelVideoInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ModelVideoInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.modelVideoInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id id
     * @return 查询结果信息
     */
    @AutoLog(value = "模板视频信息-通过id查询")
    @ApiOperation(value = "模板视频信息-通过id查询", notes = "模板视频信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ModelVideoInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ModelVideoInfo> result = new Result<>();
        ModelVideoInfo modelVideoInfo = modelVideoInfoService.getById(id);
        if (modelVideoInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(modelVideoInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
