package org.jeecg.modules.performanceappraisal.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.SysUser;
import cn.com.ctop.performanceappraisal.entity.OptimizerRoleInfo;
import cn.com.ctop.performanceappraisal.service.IOptimizerRoleInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Map;

/**
 * 优化师角色关系表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-12-17
 */
@Slf4j
@Api(tags = "优化师角色关系表")
@RestController
@RequestMapping("/ctop/optimizerRoleInfo")
public class OptimizerRoleInfoController {
    @Autowired
    private IOptimizerRoleInfoService optimizerRoleInfoService;
    @Autowired
    private ISysUserService sysUserService;

    /**
     * 分页列表查询
     *
     * @param optimizerRoleInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "优化师角色关系表-分页列表查询")
    @ApiOperation(value = "优化师角色关系表-分页列表查询", notes = "优化师角色关系表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<OptimizerRoleInfo>> queryPageList(OptimizerRoleInfo optimizerRoleInfo,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<OptimizerRoleInfo>> result = new Result<>();
        optimizerRoleInfo.setType(1);
        QueryWrapper<OptimizerRoleInfo> queryWrapper = QueryGenerator.initQueryWrapper(optimizerRoleInfo, req.getParameterMap());
        Page<OptimizerRoleInfo> page = new Page<>(pageNo, pageSize);
        IPage<OptimizerRoleInfo> pageList = optimizerRoleInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    @AutoLog(value = "优化师角色关系表-添加")
    @ApiOperation(value = "优化师角色关系表-添加", notes = "优化师角色关系表-添加")
    @PostMapping(value = "/add")
    public Result<OptimizerRoleInfo> add(String belongerId, String userId, String orgCode) {
        Result<OptimizerRoleInfo> result = new Result<OptimizerRoleInfo>();
        try {
            SysUser user = sysUserService.getById(userId);
            OptimizerRoleInfo roleInfo = new OptimizerRoleInfo();
            roleInfo.setType(2);
            roleInfo.setBelongerId(belongerId);
            roleInfo.setUserId(userId);
            roleInfo.setRealname(user.getRealname());
            roleInfo.setOrgCode(orgCode);
            roleInfo.setEnabled(1);
            optimizerRoleInfoService.save(roleInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param optimizerRoleInfo
     * @return
     */
    @AutoLog(value = "优化师角色关系表-编辑")
    @ApiOperation(value = "优化师角色关系表-编辑", notes = "优化师角色关系表-编辑")
    @PutMapping(value = "/edit")
    public Result<OptimizerRoleInfo> edit(@RequestBody OptimizerRoleInfo optimizerRoleInfo) {
        Result<OptimizerRoleInfo> result = new Result<OptimizerRoleInfo>();
        OptimizerRoleInfo optimizerRoleInfoEntity = optimizerRoleInfoService.getById(optimizerRoleInfo.getId());
        if (optimizerRoleInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = optimizerRoleInfoService.updateById(optimizerRoleInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "优化师角色关系表-通过id删除")
    @ApiOperation(value = "优化师角色关系表-通过id删除", notes = "优化师角色关系表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            optimizerRoleInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "优化师角色关系表-批量删除")
    @ApiOperation(value = "优化师角色关系表-批量删除", notes = "优化师角色关系表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<OptimizerRoleInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<OptimizerRoleInfo> result = new Result<OptimizerRoleInfo>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.optimizerRoleInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "优化师角色关系表-通过id查询")
    @ApiOperation(value = "优化师角色关系表-通过id查询", notes = "优化师角色关系表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<OptimizerRoleInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<OptimizerRoleInfo> result = new Result<OptimizerRoleInfo>();
        OptimizerRoleInfo optimizerRoleInfo = optimizerRoleInfoService.getById(id);
        if (optimizerRoleInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(optimizerRoleInfo);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 获取优化师经理的统计数据
     */
    @GetMapping("manager/detail")
    public Map<String, Object> managerDetail(@RequestParam(name = "orgCode", defaultValue = "A01") String orgCode, @RequestParam(name = "appType", defaultValue = "2") String appType) {
        return optimizerRoleInfoService.getManagerDetail(appType, orgCode);
    }
}
