package org.jeecg.modules.system.controller;

import cn.com.ctop.common.module.entity.CorpFeishuUser;
import cn.com.ctop.common.module.entity.CtopCorpWexinUser;
import cn.com.ctop.common.module.entity.SysUser;
import cn.com.ctop.common.module.service.ICorpFeishuUserService;
import cn.com.ctop.common.module.service.ICtopCorpWexinUserService;
import cn.com.ctop.common.module.utils.AesEncryptUtil;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import cn.com.feishu.provider.AuthProvider;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.exceptions.ClientException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.cp.api.WxCpOAuth2Service;
import me.chanjar.weixin.cp.api.WxCpUserService;
import me.chanjar.weixin.cp.api.impl.WxCpServiceImpl;
import me.chanjar.weixin.cp.api.impl.WxCpUserServiceImpl;
import me.chanjar.weixin.cp.bean.WxCpOauth2UserInfo;
import me.chanjar.weixin.cp.bean.WxCpUser;
import me.chanjar.weixin.cp.config.impl.WxCpDefaultConfigImpl;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.constant.CacheConstant;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.api.ISysBaseApi;
import org.jeecg.common.system.util.JwtUtil;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.*;
import org.jeecg.common.util.encryption.EncryptedString;
import org.jeecg.modules.shiro.vo.DefContants;
import org.jeecg.modules.system.entity.SysDepart;
import org.jeecg.modules.system.model.SysLoginModel;
import org.jeecg.modules.system.service.ISysDepartService;
import org.jeecg.modules.system.service.ISysDictService;
import org.jeecg.modules.system.service.ISysLogService;
import org.jeecg.modules.system.service.ISysUserService;
import org.jeecg.modules.system.util.RandImageUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * @Author scott
 * @since 2018-12-17
 */
@RestController
@RequestMapping("/sys")
@Api(tags="用户登录")
@Slf4j
public class LoginController {
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private ISysBaseApi sysBaseApi;
    @Autowired
    private ISysLogService logService;
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private ISysDepartService sysDepartService;
    @Autowired
    private ISysDictService sysDictService;
    @Autowired
    private ICorpFeishuUserService corpFeishuUserService;
    @Autowired
    private ICtopCorpWexinUserService ctopCorpWexinUserService;
    private static final String BASE_CHECK_CODES = "qwertyuiplkjhgfdsazxcvbnmQWERTYUPLKJHGFDSAZXCVBNM1234567890";
    @GetMapping("feishu/url")
    public Map<String,Object> getUrl(){
        Map<String,Object>result = new HashMap<>();
        try {
            String url = AuthProvider.getAuthUrl("");
            result.put("url",url);
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_SERVER_ERROR);
            return result;
        }
    }
    @GetMapping("/feishu/auth")
    public Result<JSONObject> feishuAuth(HttpServletRequest request,
                                         HttpServletResponse response,
                                         @RequestParam(name = "code",defaultValue = "code") String code,
                                         @RequestParam(name = "state",defaultValue = "_t") String state) throws IOException {
        Result<JSONObject> result = new Result<>();
        //1:根据code值获取用户飞书信息
        try {
            if("code".equals(code.trim())){
                JSONObject setData = new JSONObject();
                String url = AuthProvider.getAuthUrl("");
                setData.put("url",url);
                setData.put("type",1);
                result.setResult(setData);
                result.success("重定向");
                return result;
            }
            JSONObject data = AuthProvider.getFeishuUserInfo(code);
            String accessToken = data.getJSONObject("data").getString("access_token");
            String openId = data.getJSONObject("data").getString("open_id");
            //根据accessToken
            JSONObject userData = AuthProvider.getUserInfo(accessToken);
            JSONObject dataObject = userData.getJSONObject("data");
            String email = dataObject.getString("email");
            if(null == email|| "".equals(email.trim())){
                return result.error500(StatusCode.CTOP_FEISHU_EMAIL_NOT_EXIST.getDesc());
            }
            SysUser sysUser = sysUserService.getUserByEmail(email);
            if (sysUser == null) {
                return result.error500(StatusCode.CTOP_FEISHU_ACCOUNT_NOT_EXIST.getDesc());
            }
            if(sysUser.getStatus() != 1){
                return result.error500(StatusCode.CTOP_FEISHU_ACCOUNT_NOT_EXIST.getDesc());
            }
            String departId = sysDepartService.selectIdByOrdCode(sysUser.getOrgCode());
            sysUser.setDepartId(departId);
            Result<JSONObject> getResult = sysUserService.checkUserIsEffective(sysUser);
            if (!getResult.isSuccess()) {
                return result.error500(StatusCode.COMMON_ACCOUNT_ERROR.getDesc());
            }
            //用户登录信息
            userInfo(sysUser, getResult);
            sysBaseApi.addLog("用户名: " + email + ",登录成功！", CommonConstant.LOG_TYPE_1, null);
            CorpFeishuUser feishuUser = corpFeishuUserService.getByUserId(sysUser.getId());
            if (feishuUser == null) {
                feishuUser = new CorpFeishuUser();
                feishuUser.setOpenId(openId);
                feishuUser.setUserId(sysUser.getId());
                feishuUser.setEmail(sysUser.getEmail());
                feishuUser.setAvatarBig(dataObject.getString("avatar_big"));
                feishuUser.setAvatarMiddle(dataObject.getString("avatar_middle"));
                feishuUser.setAvatarThumb(dataObject.getString("avatar_thumb"));
                feishuUser.setAvatarUrl(dataObject.getString("avatar_url"));
                feishuUser.setName(dataObject.getString("name"));
                feishuUser.setMobile(dataObject.getString("mobile"));
                feishuUser.setCreateTime(new Date());
                feishuUser.setUpdateTime(new Date());
                corpFeishuUserService.save(feishuUser);
            }
            return getResult;
        } catch (Exception e) {
            e.printStackTrace();
            return result.error500(StatusCode.COMMON_SERVER_ERROR.getDesc());
        }
    }

    @RequestMapping(value = "/wxlogin", method = RequestMethod.POST)
    @ApiOperation("企业微信登录接口")
    public Result<JSONObject> wxlogin(@RequestParam(name = "code") String code) throws Exception {
        Result<JSONObject> result = new Result<>();
        val configStorage = new WxCpDefaultConfigImpl();
        configStorage.setCorpId("ww24b8a47826f5875f");
        configStorage.setAgentId(1000002);
        configStorage.setCorpSecret("MJIlySADGPlgvADnkFQPUpfZD4yV-4WN6066OgNnb0s");
        configStorage.setToken("hcst2019");
        configStorage.setAesKey("hcst2019");
        val service = new WxCpServiceImpl();

        service.setWxCpConfigStorage(configStorage);
        WxCpOAuth2Service wxCpOAuth2Service = service.getOauth2Service();
        try {
            WxCpOauth2UserInfo userInfo = wxCpOAuth2Service.getUserInfo(1000002, code);
            if (userInfo != null && userInfo.getUserId() != null) {
                WxCpUserService wxCpUserService = new WxCpUserServiceImpl(service);
                WxCpUser user = wxCpUserService.getById(userInfo.getUserId());
                String email = user.getEmail();
                SysUser sysUser = sysUserService.getUserByEmail(email);
                if (sysUser == null) {
                    result.error500("该用户未注册系统账户，请联系管理员");
                    return result;
                }
                if(sysUser.getStatus() != 1){
                    result.error500("该用户已冻结，请联系管理员");
                    return result;
                }
                String departId = sysDepartService.selectIdByOrdCode(sysUser.getOrgCode());
                sysUser.setDepartId(departId);
                result = sysUserService.checkUserIsEffective(sysUser);
                if (!result.isSuccess()) {
                    return result;
                }
                //用户登录信息
                userInfo(sysUser, result);
                sysBaseApi.addLog("用户名: " + email + ",登录成功！", CommonConstant.LOG_TYPE_1, null);
                CtopCorpWexinUser ctopCorpWexinUser = ctopCorpWexinUserService.getById(sysUser.getId());
                if (ctopCorpWexinUser == null) {
                    ctopCorpWexinUser = new CtopCorpWexinUser();
                    ctopCorpWexinUser.setUserId(sysUser.getId());
                    ctopCorpWexinUser.setWexinEmail(user.getEmail());
                    ctopCorpWexinUser.setWexinGender(user.getGender().getGenderName());
                    ctopCorpWexinUser.setWexinName(user.getName());
                    ctopCorpWexinUser.setWexinId(user.getUserId());
                    ctopCorpWexinUser.setWexinMobile(user.getMobile());
                    ctopCorpWexinUserService.save(ctopCorpWexinUser);
                }
                return result;
            }
        } catch (WxErrorException e) {
            result.error500("企业微信授权错误");
            e.printStackTrace();
        }
        return result;
    }

//    @ApiOperation("登录接口")
//    @RequestMapping(value = "/login", method = RequestMethod.POST)
//    public Result<JSONObject> login(@RequestBody SysLoginModel sysLoginModel){
//        Result<JSONObject> result = new Result<JSONObject>();
//        String username = sysLoginModel.getUsername();
//        String password = sysLoginModel.getPassword();
//        //update-begin--Author:scott  Date:20190805 for：暂时注释掉密码加密逻辑，有点问题
//        //前端密码加密，后端进行密码解密
//        //password = AesEncryptUtil.desEncrypt(sysLoginModel.getPassword().replaceAll("%2B", "\\+")).trim();//密码解密
//        //update-begin--Author:scott  Date:20190805 for：暂时注释掉密码加密逻辑，有点问题
//
//        //update-begin-author:taoyan date:20190828 for:校验验证码
//        Boolean slidered = sysLoginModel.getSlidered();
//        if (slidered!=null){ //update-begin-author:pmc 滑块验证
//            if (!slidered){
//                result.error500("滑块验证失败");
//                return result;
//            }
//        }else {
//            String captcha = sysLoginModel.getCaptcha();
//            if(captcha==null){
//                result.error500("验证码无效");
//                return result;
//            }
//            String lowerCaseCaptcha = captcha.toLowerCase();
//            String realKey = MD5Util.getMd5(lowerCaseCaptcha+sysLoginModel.getCheckKey());
//            Object checkCode = redisUtil.get(realKey);
//            if(checkCode==null || !checkCode.equals(lowerCaseCaptcha)) {
//                result.error500("验证码错误");
//                return result;
//            }
//            //update-end-author:taoyan date:20190828 for:校验验证码
//        }
//
//
//        //1. 校验用户是否有效
//        SysUser sysUser = sysUserService.getUserByName(username);
//        result = sysUserService.checkUserIsEffective(sysUser);
//        if(!result.isSuccess()) {
//            return result;
//        }
//
//        //2. 校验用户名或密码是否正确
//        String userpassword = PasswordUtil.encrypt(username, password, sysUser.getSalt());
//        String syspassword = sysUser.getPassword();
//        if (!syspassword.equals(userpassword)) {
//            result.error500("用户名或密码错误");
//            return result;
//        }
//
//        //用户登录信息
//        userInfo(sysUser, result);
//        sysBaseAPI.addLog("用户名: " + username + ",登录成功！", CommonConstant.LOG_TYPE_1, null);
//
//        return result;
//    }

    @RequestMapping(value = "/login", method = RequestMethod.POST)
    @ApiOperation("登录接口")
    public Result<JSONObject> login(@RequestBody SysLoginModel sysLoginModel) throws Exception {
        Result<JSONObject> result = new Result<>();
        String username = sysLoginModel.getUsername();
        //步骤1：TODO 前端密码加密，后端进行密码解密，防止传输密码篡改等问题，不配就直接提示密码错误，并记录日志后期进行统计分析是否锁定
        //密码解密
        String password = AesEncryptUtil.desEncrypt(sysLoginModel.getPassword()).trim();
        //1. 校验用户是否有效
        SysUser sysUser = sysUserService.getUserByName(username);
        String departId = sysDepartService.selectIdByOrdCode(sysUser.getOrgCode());
        sysUser.setDepartId(departId);
        result = sysUserService.checkUserIsEffective(sysUser);
        if (!result.isSuccess()) {
            return result;
        }

        //2. 校验用户名或密码是否正确
        String userpassword = PasswordUtil.encrypt(username, password, sysUser.getSalt());
        String syspassword = sysUser.getPassword();
        if (!syspassword.equals(userpassword)) {
            result.error500("用户名或密码错误");
            return result;
        }

        //用户登录信息
        userInfo(sysUser, result);
        sysBaseApi.addLog("用户名: " + username + ",登录成功！", CommonConstant.LOG_TYPE_1, null);
        return result;
    }

    /**
     * 退出登录
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/logout")
    public Result<Object> logout(HttpServletRequest request,HttpServletResponse response) {
        //用户退出逻辑
        String token = request.getHeader(DefContants.X_ACCESS_TOKEN);
        if(oConvertUtils.isEmpty(token)) {
            return Result.error("退出登录失败！");
        }
        String username = JwtUtil.getUsername(token);
        LoginUser sysUser = sysBaseApi.getUserByName(username);
        if(sysUser!=null) {
            sysBaseApi.addLog("用户名: "+sysUser.getRealname()+",退出成功！", CommonConstant.LOG_TYPE_1, null);
            log.info(" 用户名:  "+sysUser.getRealname()+",退出成功！ ");
            //清空用户登录Token缓存
            redisUtil.del(CommonConstant.PREFIX_USER_TOKEN + token);
            //清空用户登录Shiro权限缓存
            redisUtil.del(CommonConstant.PREFIX_USER_SHIRO_CACHE + sysUser.getId());
            //清空用户的缓存信息（包括部门信息），例如sys:cache:user::<username>
            redisUtil.del(String.format("%s::%s", CacheConstant.SYS_USERS_CACHE, sysUser.getUsername()));
            //调用shiro的logout
            SecurityUtils.getSubject().logout();
            return Result.ok("退出登录成功！");
        }else {
            return Result.error("Token无效!");
        }
    }

    /**
     * 获取访问量
     * @return
     */
    @GetMapping("loginfo")
    public Result<JSONObject> loginfo() {
        Result<JSONObject> result = new Result<JSONObject>();
        JSONObject obj = new JSONObject();
        //update-begin--Author:zhangweijian  Date:20190428 for：传入开始时间，结束时间参数
        // 获取一天的开始和结束时间
        Calendar calendar = new GregorianCalendar();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date dayStart = calendar.getTime();
        calendar.add(Calendar.DATE, 1);
        Date dayEnd = calendar.getTime();
        // 获取系统访问记录
        Long totalVisitCount = logService.findTotalVisitCount();
        obj.put("totalVisitCount", totalVisitCount);
        Long todayVisitCount = logService.findTodayVisitCount(dayStart,dayEnd);
        obj.put("todayVisitCount", todayVisitCount);
        Long todayIp = logService.findTodayIp(dayStart,dayEnd);
        //update-end--Author:zhangweijian  Date:20190428 for：传入开始时间，结束时间参数
        obj.put("todayIp", todayIp);
        result.setResult(obj);
        result.success("登录成功");
        return result;
    }

    /**
     * 获取访问量
     * @return
     */
    @GetMapping("visitInfo")
    public Result<List<Map<String,Object>>> visitInfo() {
        Result<List<Map<String,Object>>> result = new Result<List<Map<String,Object>>>();
        Calendar calendar = new GregorianCalendar();
        calendar.set(Calendar.HOUR_OF_DAY,0);
        calendar.set(Calendar.MINUTE,0);
        calendar.set(Calendar.SECOND,0);
        calendar.set(Calendar.MILLISECOND,0);
        calendar.add(Calendar.DAY_OF_MONTH, 1);
        Date dayEnd = calendar.getTime();
        calendar.add(Calendar.DAY_OF_MONTH, -7);
        Date dayStart = calendar.getTime();
        List<Map<String,Object>> list = logService.findVisitCount(dayStart, dayEnd);
        result.setResult(oConvertUtils.toLowerCasePageList(list));
        return result;
    }


    /**
     * 登陆成功选择用户当前部门
     * @param user
     * @return
     */
    @RequestMapping(value = "/selectDepart", method = RequestMethod.PUT)
    public Result<JSONObject> selectDepart(@RequestBody SysUser user) {
        Result<JSONObject> result = new Result<JSONObject>();
        String username = user.getUsername();
        if(oConvertUtils.isEmpty(username)) {
            LoginUser sysUser = (LoginUser)SecurityUtils.getSubject().getPrincipal();
            username = sysUser.getUsername();
        }
        String orgCode= user.getOrgCode();
        this.sysUserService.updateUserDepart(username, orgCode);
        SysUser sysUser = sysUserService.getUserByName(username);
        JSONObject obj = new JSONObject();
        obj.put("userInfo", sysUser);
        result.setResult(obj);
        return result;
    }

    /**
     * 短信登录接口
     *
     * @param jsonObject
     * @return
     */
    @PostMapping(value = "/sms")
    public Result<String> sms(@RequestBody JSONObject jsonObject) {
        Result<String> result = new Result<String>();
        String mobile = jsonObject.get("mobile").toString();
        //手机号模式 登录模式: "2"  注册模式: "1"
        String smsmode=jsonObject.get("smsmode").toString();
        log.info(mobile);
        if(oConvertUtils.isEmpty(mobile)){
            result.setMessage("手机号不允许为空！");
            result.setSuccess(false);
            return result;
        }
        Object object = redisUtil.get(mobile);
        if (object != null) {
            result.setMessage("验证码10分钟内，仍然有效！");
            result.setSuccess(false);
            return result;
        }

        //随机数
        String captcha = RandomUtil.randomNumbers(6);
        JSONObject obj = new JSONObject();
        obj.put("code", captcha);
        try {
            boolean b = false;
            //注册模板
            if (CommonConstant.SMS_TPL_TYPE_1.equals(smsmode)) {
                SysUser sysUser = sysUserService.getUserByPhone(mobile);
                if(sysUser!=null) {
                    result.error500(" 手机号已经注册，请直接登录！");
                    sysBaseApi.addLog("手机号已经注册，请直接登录！", CommonConstant.LOG_TYPE_1, null);
                    return result;
                }
                b = DySmsHelper.sendSms(mobile, obj, DySmsEnum.REGISTER_TEMPLATE_CODE);
            }else {
                //登录模式，校验用户有效性
                SysUser sysUser = sysUserService.getUserByPhone(mobile);
                result = sysUserService.checkUserIsEffective(sysUser);
                if(!result.isSuccess()) {
                    return result;
                }

                /**
                 * smsmode 短信模板方式  0 .登录模板、1.注册模板、2.忘记密码模板
                 */
                if (CommonConstant.SMS_TPL_TYPE_0.equals(smsmode)) {
                    //登录模板
                    b = DySmsHelper.sendSms(mobile, obj, DySmsEnum.LOGIN_TEMPLATE_CODE);
                } else if(CommonConstant.SMS_TPL_TYPE_2.equals(smsmode)) {
                    //忘记密码模板
                    b = DySmsHelper.sendSms(mobile, obj, DySmsEnum.FORGET_PASSWORD_TEMPLATE_CODE);
                }
            }

            if (b == false) {
                result.setMessage("短信验证码发送失败,请稍后重试");
                result.setSuccess(false);
                return result;
            }
            //验证码10分钟内有效
            redisUtil.set(mobile, captcha, 600);
            //update-begin--Author:scott  Date:20190812 for：issues#391
            //result.setResult(captcha);
            //update-end--Author:scott  Date:20190812 for：issues#391
            result.setSuccess(true);

        } catch (ClientException e) {
            e.printStackTrace();
            result.error500(" 短信接口未配置，请联系管理员！");
            return result;
        }
        return result;
    }


    /**
     * 手机号登录接口
     *
     * @param jsonObject
     * @return
     */
    @ApiOperation("手机号登录接口")
    @PostMapping("/phoneLogin")
    public Result<JSONObject> phoneLogin(@RequestBody JSONObject jsonObject) {
        Result<JSONObject> result = new Result<>();
        String phone = jsonObject.getString("mobile");

        //校验用户有效性
        SysUser sysUser = sysUserService.getUserByPhone(phone);
        result = sysUserService.checkUserIsEffective(sysUser);
        if(!result.isSuccess()) {
            return result;
        }

        String smscode = jsonObject.getString("captcha");
        Object code = redisUtil.get(phone);
        if (!smscode.equals(code)) {
            result.setMessage("手机验证码错误");
            return result;
        }
        //用户信息
        userInfo(sysUser, result);
        //添加日志
        sysBaseApi.addLog("用户名: " + sysUser.getUsername() + ",登录成功！", CommonConstant.LOG_TYPE_1, null);

        return result;
    }


    /**
     * 用户信息
     *
     * @param sysUser
     * @param result
     * @return
     */
    private Result<JSONObject> userInfo(SysUser sysUser, Result<JSONObject> result) {
        String syspassword = sysUser.getPassword();
        String username = sysUser.getUsername();
        // 生成token
        String token = JwtUtil.sign(username, syspassword);
        // 设置token缓存有效时间
        redisUtil.set(CommonConstant.PREFIX_USER_TOKEN + token, token);
        redisUtil.expire(CommonConstant.PREFIX_USER_TOKEN + token, JwtUtil.EXPIRE_TIME*2 / 1000);

        // 获取用户部门信息
        JSONObject obj = new JSONObject();
        List<SysDepart> departs = sysDepartService.queryUserDeparts(sysUser.getId());
        obj.put("departs", departs);
        if (departs == null || departs.size() == 0) {
            obj.put("multi_depart", 0);
        } else if (departs.size() == 1) {
            sysUserService.updateUserDepart(username, departs.get(0).getOrgCode());
            obj.put("multi_depart", 1);
        } else {
            obj.put("multi_depart", 2);
        }
        obj.put("token", token);
        obj.put("userInfo", sysUser);
        obj.put("sysAllDictItems", sysDictService.queryAllDictItems());
        result.setResult(obj);
        result.success("登录成功");
        return result;
    }

    /**
     * 获取加密字符串
     * @return
     */
    @GetMapping(value = "/getEncryptedString")
    public Result<Map<String,String>> getEncryptedString(){
        Result<Map<String,String>> result = new Result<>();
        Map<String,String> map = new HashMap<>();
        map.put("key", EncryptedString.key);
        map.put("iv",EncryptedString.iv);
        result.setResult(map);
        return result;
    }

    /**
     * 后台生成图形验证码 ：有效
     * @param response
     * @param key
     */
    @ApiOperation("获取验证码")
    @GetMapping(value = "/randomImage/{key}")
    public Result<String> randomImage(HttpServletResponse response,@PathVariable String key){
        Result<String> res = new Result<String>();
        try {
            String code = RandomUtil.randomString(BASE_CHECK_CODES,4);
            String lowerCaseCode = code.toLowerCase();
            String realKey = MD5Util.getMd5(lowerCaseCode);
            redisUtil.set(realKey, lowerCaseCode, 60);
            String base64 = RandImageUtil.generate(code);
            res.setSuccess(true);
            res.setResult(base64);
        } catch (Exception e) {
            res.error500("获取验证码出错"+e.getMessage());
            e.printStackTrace();
        }
        return res;
    }

    /**
     * app登录
     * @param sysLoginModel
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/mLogin", method = RequestMethod.POST)
    public Result<JSONObject> mLogin(@RequestBody SysLoginModel sysLoginModel) throws Exception {
        Result<JSONObject> result = new Result<JSONObject>();
        String username = sysLoginModel.getUsername();
        String password = sysLoginModel.getPassword();

        //1. 校验用户是否有效
        SysUser sysUser = sysUserService.getUserByName(username);
        result = sysUserService.checkUserIsEffective(sysUser);
        if(!result.isSuccess()) {
            return result;
        }

        //2. 校验用户名或密码是否正确
        String userpassword = PasswordUtil.encrypt(username, password, sysUser.getSalt());
        String syspassword = sysUser.getPassword();
        if (!syspassword.equals(userpassword)) {
            result.error500("用户名或密码错误");
            return result;
        }

        String orgCode = sysUser.getOrgCode();
        if(oConvertUtils.isEmpty(orgCode)) {
            //如果当前用户无选择部门 查看部门关联信息
            List<SysDepart> departs = sysDepartService.queryUserDeparts(sysUser.getId());
            if (departs == null || departs.size() == 0) {
                result.error500("用户暂未归属部门,不可登录!");
                return result;
            }
            orgCode = departs.get(0).getOrgCode();
            sysUser.setOrgCode(orgCode);
            this.sysUserService.updateUserDepart(username, orgCode);
        }
        JSONObject obj = new JSONObject();
        //用户登录信息
        obj.put("userInfo", sysUser);

        // 生成token
        String token = JwtUtil.sign(username, syspassword);
        // 设置超时时间
        redisUtil.set(CommonConstant.PREFIX_USER_TOKEN + token, token);
        redisUtil.expire(CommonConstant.PREFIX_USER_TOKEN + token, JwtUtil.EXPIRE_TIME*2 / 1000);
        //token 信息
        obj.put("token", token);
        result.setResult(obj);
        result.setSuccess(true);
        result.setCode(200);
        sysBaseApi.addLog("用户名: " + username + ",登录成功[移动端]！", CommonConstant.LOG_TYPE_1, null);
        return result;
    }

    /**
     * 图形验证码
     * @param sysLoginModel
     * @return
     */
    @RequestMapping(value = "/checkCaptcha", method = RequestMethod.POST)
    public Result<?> checkCaptcha(@RequestBody SysLoginModel sysLoginModel){
        String captcha = sysLoginModel.getCaptcha();
        String checkKey = sysLoginModel.getCheckKey();
        if(captcha==null){
            return Result.error("验证码无效");
        }
        String lowerCaseCaptcha = captcha.toLowerCase();
        String realKey = MD5Util.getMd5(lowerCaseCaptcha+checkKey);
        Object checkCode = redisUtil.get(realKey);
        if(checkCode==null || !checkCode.equals(lowerCaseCaptcha)) {
            return Result.error("验证码错误");
        }
        return Result.ok();
    }

}
