package org.jeecg.modules.wps.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.wps.entity.WpsFileVersion;
import org.jeecg.modules.wps.service.IWpsFileVersionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * wps文件版本号
 * @author jeecg-boot
 * @date   2020-06-01
 * @version V1.0
 */
@Slf4j
@Api(tags="wps文件版本号")
@RestController
@RequestMapping("/ctop/wpsFileVersion")
public class WpsFileVersionController {
	@Autowired
	private IWpsFileVersionService wpsFileVersionService;

	/**
	  * 分页列表查询
	 * @param wpsFileVersion
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "wps文件版本号-分页列表查询")
	@ApiOperation(value="wps文件版本号-分页列表查询", notes="wps文件版本号-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<WpsFileVersion>> queryPageList(WpsFileVersion wpsFileVersion,
													   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													   HttpServletRequest req) {
		Result<IPage<WpsFileVersion>> result = new Result<>();
		QueryWrapper<WpsFileVersion> queryWrapper = QueryGenerator.initQueryWrapper(wpsFileVersion, req.getParameterMap());
		Page<WpsFileVersion> page = new Page<>(pageNo, pageSize);
		IPage<WpsFileVersion> pageList = wpsFileVersionService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param wpsFileVersion
	 * @return
	 */
	@AutoLog(value = "wps文件版本号-添加")
	@ApiOperation(value="wps文件版本号-添加", notes="wps文件版本号-添加")
	@PostMapping(value = "/add")
	public Result<WpsFileVersion> add(@RequestBody WpsFileVersion wpsFileVersion) {
		Result<WpsFileVersion> result = new Result<>();
		try {
			wpsFileVersionService.save(wpsFileVersion);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param wpsFileVersion
	 * @return
	 */
	@AutoLog(value = "wps文件版本号-编辑")
	@ApiOperation(value="wps文件版本号-编辑", notes="wps文件版本号-编辑")
	@PutMapping(value = "/edit")
	public Result<WpsFileVersion> edit(@RequestBody WpsFileVersion wpsFileVersion) {
		Result<WpsFileVersion> result = new Result<>();
		WpsFileVersion wpsFileVersionEntity = wpsFileVersionService.getById(wpsFileVersion.getId());
		if(wpsFileVersionEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = wpsFileVersionService.updateById(wpsFileVersion);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps文件版本号-通过id删除")
	@ApiOperation(value="wps文件版本号-通过id删除", notes="wps文件版本号-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Object> delete(@RequestParam(name="id",required=true) String id) {
		try {
			wpsFileVersionService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "wps文件版本号-批量删除")
	@ApiOperation(value="wps文件版本号-批量删除", notes="wps文件版本号-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<WpsFileVersion> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<WpsFileVersion> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.wpsFileVersionService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps文件版本号-通过id查询")
	@ApiOperation(value="wps文件版本号-通过id查询", notes="wps文件版本号-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<WpsFileVersion> queryById(@RequestParam(name="id",required=true) String id) {
		Result<WpsFileVersion> result = new Result<>();
		WpsFileVersion wpsFileVersion = wpsFileVersionService.getById(id);
		if(wpsFileVersion==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(wpsFileVersion);
			result.setSuccess(true);
		}
		return result;
	}
}
