package org.jeecg.modules.wps.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.wps.entity.WpsFileWatermark;
import org.jeecg.modules.wps.service.IWpsFileWatermarkService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * wps文件水印
 * @author jeecg-boot
 * @date   2020-06-01
 * @version V1.0
 */
@Slf4j
@Api(tags="wps文件水印")
@RestController
@RequestMapping("/ctop/wpsFileWatermark")
public class WpsFileWatermarkController {
	@Autowired
	private IWpsFileWatermarkService wpsFileWatermarkService;

	/**
	  * 分页列表查询
	 * @param wpsFileWatermark
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "wps文件水印-分页列表查询")
	@ApiOperation(value="wps文件水印-分页列表查询", notes="wps文件水印-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<WpsFileWatermark>> queryPageList(WpsFileWatermark wpsFileWatermark,
														 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														 HttpServletRequest req) {
		Result<IPage<WpsFileWatermark>> result = new Result<>();
		QueryWrapper<WpsFileWatermark> queryWrapper = QueryGenerator.initQueryWrapper(wpsFileWatermark, req.getParameterMap());
		Page<WpsFileWatermark> page = new Page<WpsFileWatermark>(pageNo, pageSize);
		IPage<WpsFileWatermark> pageList = wpsFileWatermarkService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param wpsFileWatermark
	 * @return
	 */
	@AutoLog(value = "wps文件水印-添加")
	@ApiOperation(value="wps文件水印-添加", notes="wps文件水印-添加")
	@PostMapping(value = "/add")
	public Result<WpsFileWatermark> add(@RequestBody WpsFileWatermark wpsFileWatermark) {
		Result<WpsFileWatermark> result = new Result<>();
		try {
			wpsFileWatermarkService.save(wpsFileWatermark);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param wpsFileWatermark
	 * @return
	 */
	@AutoLog(value = "wps文件水印-编辑")
	@ApiOperation(value="wps文件水印-编辑", notes="wps文件水印-编辑")
	@PutMapping(value = "/edit")
	public Result<WpsFileWatermark> edit(@RequestBody WpsFileWatermark wpsFileWatermark) {
		Result<WpsFileWatermark> result = new Result<>();
		WpsFileWatermark wpsFileWatermarkEntity = wpsFileWatermarkService.getById(wpsFileWatermark.getId());
		if(wpsFileWatermarkEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = wpsFileWatermarkService.updateById(wpsFileWatermark);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps文件水印-通过id删除")
	@ApiOperation(value="wps文件水印-通过id删除", notes="wps文件水印-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Object> delete(@RequestParam(name="id",required=true) String id) {
		try {
			wpsFileWatermarkService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "wps文件水印-批量删除")
	@ApiOperation(value="wps文件水印-批量删除", notes="wps文件水印-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<WpsFileWatermark> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<WpsFileWatermark> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.wpsFileWatermarkService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps文件水印-通过id查询")
	@ApiOperation(value="wps文件水印-通过id查询", notes="wps文件水印-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<WpsFileWatermark> queryById(@RequestParam(name="id",required=true) String id) {
		Result<WpsFileWatermark> result = new Result<>();
		WpsFileWatermark wpsFileWatermark = wpsFileWatermarkService.getById(id);
		if(wpsFileWatermark==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(wpsFileWatermark);
			result.setSuccess(true);
		}
		return result;
	}
}
