package org.jeecg.modules.wps.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.wps.entity.WpsUserAcl;
import org.jeecg.modules.wps.service.IWpsUserAclService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * wps用户权限
 * @author jeecg-boot
 * @date   2020-06-01
 * @version V1.0
 */
@Slf4j
@Api(tags="wps用户权限")
@RestController
@RequestMapping("/ctop/wpsUserAcl")
public class WpsUserAclController {
	@Autowired
	private IWpsUserAclService wpsUserAclService;

	/**
	  * 分页列表查询
	 * @param wpsUserAcl
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "wps用户权限-分页列表查询")
	@ApiOperation(value="wps用户权限-分页列表查询", notes="wps用户权限-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<WpsUserAcl>> queryPageList(WpsUserAcl wpsUserAcl,
												   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
												   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
												   HttpServletRequest req) {
		Result<IPage<WpsUserAcl>> result = new Result<>();
		QueryWrapper<WpsUserAcl> queryWrapper = QueryGenerator.initQueryWrapper(wpsUserAcl, req.getParameterMap());
		Page<WpsUserAcl> page = new Page<>(pageNo, pageSize);
		IPage<WpsUserAcl> pageList = wpsUserAclService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param wpsUserAcl
	 * @return
	 */
	@AutoLog(value = "wps用户权限-添加")
	@ApiOperation(value="wps用户权限-添加", notes="wps用户权限-添加")
	@PostMapping(value = "/add")
	public Result<WpsUserAcl> add(@RequestBody WpsUserAcl wpsUserAcl) {
		Result<WpsUserAcl> result = new Result<>();
		try {
			wpsUserAclService.save(wpsUserAcl);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param wpsUserAcl
	 * @return
	 */
	@AutoLog(value = "wps用户权限-编辑")
	@ApiOperation(value="wps用户权限-编辑", notes="wps用户权限-编辑")
	@PutMapping(value = "/edit")
	public Result<WpsUserAcl> edit(@RequestBody WpsUserAcl wpsUserAcl) {
		Result<WpsUserAcl> result = new Result<>();
		WpsUserAcl wpsUserAclEntity = wpsUserAclService.getById(wpsUserAcl.getId());
		if(wpsUserAclEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = wpsUserAclService.updateById(wpsUserAcl);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps用户权限-通过id删除")
	@ApiOperation(value="wps用户权限-通过id删除", notes="wps用户权限-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Object> delete(@RequestParam(name="id",required=true) String id) {
		try {
			wpsUserAclService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "wps用户权限-批量删除")
	@ApiOperation(value="wps用户权限-批量删除", notes="wps用户权限-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<WpsUserAcl> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<WpsUserAcl> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.wpsUserAclService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps用户权限-通过id查询")
	@ApiOperation(value="wps用户权限-通过id查询", notes="wps用户权限-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<WpsUserAcl> queryById(@RequestParam(name="id",required=true) String id) {
		Result<WpsUserAcl> result = new Result<>();
		WpsUserAcl wpsUserAcl = wpsUserAclService.getById(id);
		if(wpsUserAcl==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(wpsUserAcl);
			result.setSuccess(true);
		}
		return result;
	}
}
