package cn.com.ctop.alarm.modules.controller;

import cn.com.ctop.alarm.modules.entity.AlarmEventMetric;
import cn.com.ctop.alarm.modules.service.IAlarmEventMetricService;
import cn.com.ctop.common.module.annotation.AutoLog;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 制定详细规则表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-10-29
 */
@Slf4j
@Api(tags = "制定详细规则表")
@RestController
@RequestMapping("/alarm/alarmEventMetric")
public class AlarmEventMetricController {
    @Autowired
    private IAlarmEventMetricService alarmEventMetricService;

    /**
     * 分页列表查询
     *
     * @param alarmEventMetric
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "制定详细规则表-分页列表查询")
    @ApiOperation(value = "制定详细规则表-分页列表查询", notes = "制定详细规则表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AlarmEventMetric>> queryPageList(AlarmEventMetric alarmEventMetric,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<AlarmEventMetric>> result = new Result<>();
        QueryWrapper<AlarmEventMetric> queryWrapper = QueryGenerator.initQueryWrapper(alarmEventMetric, req.getParameterMap());
        Page<AlarmEventMetric> page = new Page<AlarmEventMetric>(pageNo, pageSize);
        IPage<AlarmEventMetric> pageList = alarmEventMetricService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param alarmEventMetric
     * @return
     */
    @AutoLog(value = "制定详细规则表-添加")
    @ApiOperation(value = "制定详细规则表-添加", notes = "制定详细规则表-添加")
    @PostMapping(value = "/add")
    public Result<AlarmEventMetric> add(@RequestBody AlarmEventMetric alarmEventMetric) {
        Result<AlarmEventMetric> result = new Result<>();
        try {
            alarmEventMetricService.save(alarmEventMetric);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param alarmEventMetric
     * @return
     */
    @AutoLog(value = "制定详细规则表-编辑")
    @ApiOperation(value = "制定详细规则表-编辑", notes = "制定详细规则表-编辑")
    @PutMapping(value = "/edit")
    public Result<AlarmEventMetric> edit(@RequestBody AlarmEventMetric alarmEventMetric) {
        Result<AlarmEventMetric> result = new Result<AlarmEventMetric>();
        AlarmEventMetric alarmEventMetricEntity = alarmEventMetricService.getById(alarmEventMetric.getEventMetricId());
        if (alarmEventMetricEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = alarmEventMetricService.updateById(alarmEventMetric);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "制定详细规则表-通过id删除")
    @ApiOperation(value = "制定详细规则表-通过id删除", notes = "制定详细规则表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            alarmEventMetricService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "制定详细规则表-批量删除")
    @ApiOperation(value = "制定详细规则表-批量删除", notes = "制定详细规则表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AlarmEventMetric> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<AlarmEventMetric> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.alarmEventMetricService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "制定详细规则表-通过id查询")
    @ApiOperation(value = "制定详细规则表-通过id查询", notes = "制定详细规则表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AlarmEventMetric> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AlarmEventMetric> result = new Result<>();
        AlarmEventMetric alarmEventMetric = alarmEventMetricService.getById(id);
        if (alarmEventMetric == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(alarmEventMetric);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AlarmEventMetric> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AlarmEventMetric alarmEventMetric = JSON.parseObject(deString, AlarmEventMetric.class);
                queryWrapper = QueryGenerator.initQueryWrapper(alarmEventMetric, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AlarmEventMetric> pageList = alarmEventMetricService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "制定详细规则表列表");
        mv.addObject(NormalExcelConstants.CLASS, AlarmEventMetric.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("制定详细规则表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AlarmEventMetric> listAlarmEventMetrics = ExcelImportUtil.importExcel(file.getInputStream(), AlarmEventMetric.class, params);
                alarmEventMetricService.saveBatch(listAlarmEventMetrics);
                return Result.ok("文件导入成功！数据行数:" + listAlarmEventMetrics.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
