package cn.com.ctop.alarm.modules.controller;

import cn.com.ctop.alarm.modules.entity.AlarmMetricValue;
import cn.com.ctop.alarm.modules.service.IAlarmMetricValueService;
import cn.com.ctop.common.module.annotation.AutoLog;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 指标字段记录表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-10-29
 */
@Slf4j
@Api(tags = "指标字段记录表")
@RestController
@RequestMapping("/alarm/alarmMetricValue")
public class AlarmMetricValueController {
    @Autowired
    private IAlarmMetricValueService alarmMetricValueService;

    /**
     * 分页列表查询
     *
     * @param alarmMetricValue
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "指标字段记录表-分页列表查询")
    @ApiOperation(value = "指标字段记录表-分页列表查询", notes = "指标字段记录表-分页列表查询")
    @GetMapping(value = "/queryPageList")
    public Result<IPage<AlarmMetricValue>> queryPageList(AlarmMetricValue alarmMetricValue,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<AlarmMetricValue>> result = new Result<>();
        QueryWrapper<AlarmMetricValue> queryWrapper = QueryGenerator.initQueryWrapper(alarmMetricValue, req.getParameterMap());
        Page<AlarmMetricValue> page = new Page<AlarmMetricValue>(pageNo, pageSize);
        IPage<AlarmMetricValue> pageList = alarmMetricValueService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @AutoLog(value = "指标字段记录表")
    @ApiOperation(value = "指标字段记录表-分页列表查询", notes = "指标字段记录表")
    @GetMapping(value = "/queryParentChildList")
    public Result<Object> queryParentChildList(AlarmMetricValue alarmMetricValue, HttpServletRequest req) {
        Result<Object> result = null;
        try {
            result = alarmMetricValueService.queryParentChildList(alarmMetricValue);
        } catch (Exception e) {
            log.error("查询指标数据异常：", e);
            result = Result.error("查询失败");
        }
        return result;
    }

    /**
     * 添加
     *
     * @param alarmMetricValue
     * @return
     */
    @AutoLog(value = "指标字段记录表-添加")
    @ApiOperation(value = "指标字段记录表-添加", notes = "指标字段记录表-添加")
    @PostMapping(value = "/add")
    public Result<AlarmMetricValue> add(@RequestBody AlarmMetricValue alarmMetricValue) {
        Result<AlarmMetricValue> result = new Result<>();
        try {
            alarmMetricValueService.save(alarmMetricValue);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param alarmMetricValue
     * @return
     */
    @AutoLog(value = "指标字段记录表-编辑")
    @ApiOperation(value = "指标字段记录表-编辑", notes = "指标字段记录表-编辑")
    @GetMapping(value = "/edit")
    public Result<AlarmMetricValue> edit(@RequestBody AlarmMetricValue alarmMetricValue) {
        Result<AlarmMetricValue> result = new Result<AlarmMetricValue>();
        AlarmMetricValue alarmMetricValueEntity = alarmMetricValueService.getById(alarmMetricValue.getMetricValueId());
        if (alarmMetricValueEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = alarmMetricValueService.updateById(alarmMetricValue);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "指标字段记录表-通过id删除")
    @ApiOperation(value = "指标字段记录表-通过id删除", notes = "指标字段记录表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            alarmMetricValueService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "指标字段记录表-批量删除")
    @ApiOperation(value = "指标字段记录表-批量删除", notes = "指标字段记录表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AlarmMetricValue> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<AlarmMetricValue> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.alarmMetricValueService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "指标字段记录表-通过id查询")
    @ApiOperation(value = "指标字段记录表-通过id查询", notes = "指标字段记录表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AlarmMetricValue> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AlarmMetricValue> result = new Result<>();
        AlarmMetricValue alarmMetricValue = alarmMetricValueService.getById(id);
        if (alarmMetricValue == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(alarmMetricValue);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AlarmMetricValue> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AlarmMetricValue alarmMetricValue = JSON.parseObject(deString, AlarmMetricValue.class);
                queryWrapper = QueryGenerator.initQueryWrapper(alarmMetricValue, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AlarmMetricValue> pageList = alarmMetricValueService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "指标字段记录表列表");
        mv.addObject(NormalExcelConstants.CLASS, AlarmMetricValue.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("指标字段记录表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AlarmMetricValue> listAlarmMetricValues = ExcelImportUtil.importExcel(file.getInputStream(), AlarmMetricValue.class, params);
                alarmMetricValueService.saveBatch(listAlarmMetricValues);
                return Result.ok("文件导入成功！数据行数:" + listAlarmMetricValues.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
