package cn.com.ctop.alarm.modules.controller;

import cn.com.ctop.alarm.modules.entity.RuleAccountTemplate;
import cn.com.ctop.alarm.modules.entity.RuleTemplate;
import cn.com.ctop.alarm.modules.entity.vo.TemplateAppliedVo;
import cn.com.ctop.alarm.modules.service.IRuleAccountTemplateService;
import cn.com.ctop.alarm.modules.service.IRuleAccountThresholdService;
import cn.com.ctop.alarm.modules.service.IRuleTemplateService;
import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.service.ISysRoleExtService;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 规则模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-11-15
 */
@Slf4j
@Api(tags = "规则模板")
@RestController
@RequestMapping("/ruleAccountTemplate")
public class RuleAccountTemplateController {
    @Autowired
    private IRuleAccountTemplateService ruleAccountTemplateService;
    @Autowired
    private IRuleTemplateService templateService;
    @Autowired
    private IRuleAccountThresholdService accountThresholdService;

    @Autowired
    private ISysRoleExtService sysRoleService;

    /**
     * 查询账户下关联的模板
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/getTemplateApplied")
    public Result<PageInfo<TemplateAppliedVo>> getTemplateApplied(int pageNo, int pageSize, Long accountId, String projectName, String userName) {
        Result<PageInfo<TemplateAppliedVo>> result = new Result<>();
        try {
            LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
            String roleCode = sysRoleService.getRoleCodeByUserId(sysUser.getId());
            Map<String, Object> requestMap = new JSONObject();
            if (!"admin".equals(roleCode)) {
                requestMap.put("userId", sysUser.getId());
            }
            if (!Check.isNull(accountId)) {
                requestMap.put("accountId", accountId);
            }

            if (!Check.isNull(projectName)) {
                requestMap.put("projectName", projectName);
            }
            if (!Check.isNull(userName)) {
                requestMap.put("userName", userName);
            }
            PageHelper.startPage(pageNo, pageSize);
            List<TemplateAppliedVo> list = ruleAccountTemplateService.getTemplateApplied(requestMap);

            PageInfo<TemplateAppliedVo> pageInfo = new PageInfo<>(list);
            result.setSuccess(true);
            result.setResult(pageInfo);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    @GetMapping(value = "/checkAccountTemplate")
    public Result<JSONObject> checkAccountTemplate(Long accountId) {
        Result<JSONObject> result = new Result<>();
        JSONObject returnJson = new JSONObject();
        QueryWrapper<RuleAccountTemplate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        queryWrapper.orderByDesc("create_time");
        queryWrapper.last("limit 1");
        RuleAccountTemplate ruleAccountTemplate = ruleAccountTemplateService.getOne(queryWrapper);
        if (!Check.isNull(ruleAccountTemplate)) {
            RuleTemplate ruleTemplate = templateService.getById(ruleAccountTemplate.getTemplateId());
            returnJson.put("isApplication", true);
            if (!Check.isNull(ruleTemplate)) {
                returnJson.put("templateName", ruleTemplate.getTemplateName());
            }
        } else {
            returnJson.put("isApplication", false);
        }

        result.setSuccess(true);
        result.setResult(returnJson);
        return result;
    }


    /**
     * 推送 模板应用到某个账户
     *
     * @param ruleAccountTemplate
     * @return
     */
    @PostMapping(value = "/pushTemplateToAccount")
    public Result pushTemplateToAccount(@RequestBody RuleAccountTemplate ruleAccountTemplate) {
        Result result = new Result<>();
        try {
            if (Check.isNull(ruleAccountTemplate.getAccountId())) {
                throw new Exception("请选择需要推送的账户");
            }

            if (Check.isNull(ruleAccountTemplate.getTemplateId())) {
                throw new Exception("请选择需要推送的模板");
            }

            JSONObject returnJson = ruleAccountTemplateService.pushTemplateToAccount(ruleAccountTemplate.getAccountId(), ruleAccountTemplate.getTemplateId());
            result.setSuccess(true);
            result.setResult(returnJson);
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "规则模板-通过id删除")
    @ApiOperation(value = "规则模板-通过id删除", notes = "规则模板-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            RuleAccountTemplate accountTemplate = ruleAccountTemplateService.getById(id);
            if (!Check.isNull(accountTemplate)) {
                Map<String, Object> deleteMap = new HashMap<>();
                deleteMap.put("account_id", accountTemplate.getAccountId());
                accountThresholdService.removeByMap(deleteMap);
                ruleAccountTemplateService.removeById(id);
            }
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }


    /**
     * 分页列表查询
     *
     * @param ruleAccountTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "规则模板-分页列表查询")
    @ApiOperation(value = "规则模板-分页列表查询", notes = "规则模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<RuleAccountTemplate>> queryPageList(RuleAccountTemplate ruleAccountTemplate,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<RuleAccountTemplate>> result = new Result<>();
        QueryWrapper<RuleAccountTemplate> queryWrapper = QueryGenerator.initQueryWrapper(ruleAccountTemplate, req.getParameterMap());
        Page<RuleAccountTemplate> page = new Page<RuleAccountTemplate>(pageNo, pageSize);
        IPage<RuleAccountTemplate> pageList = ruleAccountTemplateService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    @GetMapping(value = "/getRelationAccountById")
    public Result<JSONObject> getRelationAccountById(Long id) {
        Result<JSONObject> result = new Result<>();
        JSONObject returnJson = new JSONObject();
        QueryWrapper<RuleAccountTemplate> accountTemplateQueryWrapper = new QueryWrapper<>();
        accountTemplateQueryWrapper.eq("template_id", id);
        List<RuleAccountTemplate> list = ruleAccountTemplateService.list(accountTemplateQueryWrapper);
        returnJson.put("relationCount", list.size());
        result.setSuccess(true);
        result.setResult(returnJson);
        return result;
    }


    /**
     * 添加
     *
     * @param ruleAccountTemplate
     * @return
     */
    @AutoLog(value = "规则模板-添加")
    @ApiOperation(value = "规则模板-添加", notes = "规则模板-添加")
    @PostMapping(value = "/add")
    public Result<RuleAccountTemplate> add(@RequestBody RuleAccountTemplate ruleAccountTemplate) {
        Result<RuleAccountTemplate> result = new Result<>();
        try {
            ruleAccountTemplateService.save(ruleAccountTemplate);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param ruleAccountTemplate
     * @return
     */
    @AutoLog(value = "规则模板-编辑")
    @ApiOperation(value = "规则模板-编辑", notes = "规则模板-编辑")
    @PutMapping(value = "/edit")
    public Result<RuleAccountTemplate> edit(@RequestBody RuleAccountTemplate ruleAccountTemplate) {
        Result<RuleAccountTemplate> result = new Result<RuleAccountTemplate>();
        RuleAccountTemplate ruleAccountTemplateEntity = ruleAccountTemplateService.getById(ruleAccountTemplate.getId());
        if (ruleAccountTemplateEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = ruleAccountTemplateService.updateById(ruleAccountTemplate);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }


    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "规则模板-批量删除")
    @ApiOperation(value = "规则模板-批量删除", notes = "规则模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<RuleAccountTemplate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<RuleAccountTemplate> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.ruleAccountTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "规则模板-通过id查询")
    @ApiOperation(value = "规则模板-通过id查询", notes = "规则模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<RuleAccountTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<RuleAccountTemplate> result = new Result<>();
        RuleAccountTemplate ruleAccountTemplate = ruleAccountTemplateService.getById(id);
        if (ruleAccountTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(ruleAccountTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<RuleAccountTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                RuleAccountTemplate ruleAccountTemplate = JSON.parseObject(deString, RuleAccountTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(ruleAccountTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<RuleAccountTemplate> pageList = ruleAccountTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "规则模板列表");
        mv.addObject(NormalExcelConstants.CLASS, RuleAccountTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("规则模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<RuleAccountTemplate> listRuleAccountTemplates = ExcelImportUtil.importExcel(file.getInputStream(), RuleAccountTemplate.class, params);
                ruleAccountTemplateService.saveBatch(listRuleAccountTemplates);
                return Result.ok("文件导入成功！数据行数:" + listRuleAccountTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
