package cn.com.ctop.alarm.modules.controller;

import cn.com.ctop.alarm.modules.entity.RuleGroup;
import cn.com.ctop.alarm.modules.entity.RuleTemplate;
import cn.com.ctop.alarm.modules.service.IRuleAccountThresholdService;
import cn.com.ctop.alarm.modules.service.IRuleBaseService;
import cn.com.ctop.alarm.modules.service.IRuleGroupService;
import cn.com.ctop.alarm.modules.service.IRuleTemplateService;
import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * 规则组
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-11-15
 */
@Slf4j
@Api(tags = "规则组")
@RestController
@RequestMapping("/ruleGroup")
public class RuleGroupController {
    @Autowired
    private IRuleGroupService ruleGroupService;
    @Autowired
    private IRuleBaseService ruleBaseService;
    @Autowired
    private IRuleAccountThresholdService ruleAccountThresholdService;
    @Autowired
    private IRuleTemplateService ruleTemplateService;


    /**
     * 创建规则组
     *
     * @param requestJson
     * @return
     */

    @PostMapping(value = "/createRuleGroup")
    public Result createRuleGroup(@RequestBody JSONObject requestJson) {
        Result result = new Result<>();
        try {
            if (Check.isNull(requestJson)) {
                throw new Exception("请输出需要创建的参数");
            }
            JSONObject returnJson = ruleGroupService.createRuleGroup(requestJson);
            if (returnJson.getInteger("code") == 0) {
                result.setSuccess(true);
                result.setMessage(returnJson.getString("message"));
            } else {
                result.setSuccess(false);
                result.setMessage(returnJson.getString("message"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "规则组-通过id删除")
    @ApiOperation(value = "规则组-通过id删除", notes = "规则组-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            RuleGroup ruleGroup = ruleGroupService.getById(id);
            if (!Check.isNull(ruleGroup)) {
                JSONArray ruleIds = JSONArray.parseArray(ruleGroup.getRuleIds());
                for (int i = 0; i < ruleIds.size(); i++) {
                    Long ruleId = ruleIds.getLong(i);
                    ruleBaseService.removeById(ruleId);
                    Map<String, Object> deleteMap = new HashMap<>();
                    deleteMap.put("rule_id", ruleId);
                    ruleAccountThresholdService.removeByMap(deleteMap);
                }
                Long templateId = ruleGroup.getTemplateId();
                RuleTemplate ruleTemplate = ruleTemplateService.getById(templateId);
                if (!Check.isNull(ruleTemplate)) {
                    JSONArray newGroupIds = new JSONArray();
                    JSONArray groupIds = JSONArray.parseArray(ruleTemplate.getGroupIds());
                    Iterator<Object> o = groupIds.iterator();
                    while (o.hasNext()) {
                        String groupId = String.valueOf(o.next());
                        if (groupId.equals(id)) {
                            continue;
                        }
                        newGroupIds.add(Long.valueOf(groupId));
                    }
                    ruleTemplate.setGroupIds(newGroupIds.toJSONString());
                    ruleTemplateService.updateById(ruleTemplate);
                }
                ruleGroupService.removeById(id);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }


    /**
     * 分页列表查询
     *
     * @param ruleGroup
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "规则组-分页列表查询")
    @ApiOperation(value = "规则组-分页列表查询", notes = "规则组-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<RuleGroup>> queryPageList(RuleGroup ruleGroup,
                                                  @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                  HttpServletRequest req) {
        Result<IPage<RuleGroup>> result = new Result<>();
        QueryWrapper<RuleGroup> queryWrapper = QueryGenerator.initQueryWrapper(ruleGroup, req.getParameterMap());
        Page<RuleGroup> page = new Page<RuleGroup>(pageNo, pageSize);
        IPage<RuleGroup> pageList = ruleGroupService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param ruleGroup
     * @return
     */
    @AutoLog(value = "规则组-添加")
    @ApiOperation(value = "规则组-添加", notes = "规则组-添加")
    @PostMapping(value = "/add")
    public Result<RuleGroup> add(@RequestBody RuleGroup ruleGroup) {
        Result<RuleGroup> result = new Result<>();
        try {
            ruleGroupService.save(ruleGroup);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    @GetMapping(value = "/checkRules")
    public Result<RuleGroup> checkRules() {
        Result<RuleGroup> result = new Result<>();
        try {
            ruleGroupService.checkRules();
            result.success("成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param ruleGroup
     * @return
     */
    @AutoLog(value = "规则组-编辑")
    @ApiOperation(value = "规则组-编辑", notes = "规则组-编辑")
    @PutMapping(value = "/edit")
    public Result<RuleGroup> edit(@RequestBody RuleGroup ruleGroup) {
        Result<RuleGroup> result = new Result<RuleGroup>();
        RuleGroup ruleGroupEntity = ruleGroupService.getById(ruleGroup.getId());
        if (ruleGroupEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = ruleGroupService.updateById(ruleGroup);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }


    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "规则组-批量删除")
    @ApiOperation(value = "规则组-批量删除", notes = "规则组-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<RuleGroup> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<RuleGroup> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.ruleGroupService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "规则组-通过id查询")
    @ApiOperation(value = "规则组-通过id查询", notes = "规则组-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<RuleGroup> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<RuleGroup> result = new Result<>();
        RuleGroup ruleGroup = ruleGroupService.getById(id);
        if (ruleGroup == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(ruleGroup);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<RuleGroup> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                RuleGroup ruleGroup = JSON.parseObject(deString, RuleGroup.class);
                queryWrapper = QueryGenerator.initQueryWrapper(ruleGroup, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<RuleGroup> pageList = ruleGroupService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "规则组列表");
        mv.addObject(NormalExcelConstants.CLASS, RuleGroup.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("规则组列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<RuleGroup> listRuleGroups = ExcelImportUtil.importExcel(file.getInputStream(), RuleGroup.class, params);
                ruleGroupService.saveBatch(listRuleGroups);
                return Result.ok("文件导入成功！数据行数:" + listRuleGroups.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
