package cn.com.ctop.alarm.modules.controller;

import cn.com.ctop.alarm.modules.entity.RuleGroup;
import cn.com.ctop.alarm.modules.entity.RuleTemplate;
import cn.com.ctop.alarm.modules.service.*;
import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 规则模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-11-15
 */
@Slf4j
@Api(tags = "规则模板")
@RestController
@RequestMapping("/ruleTemplate")
public class RuleTemplateController {
    @Autowired
    private IRuleTemplateService ruleTemplateService;
    @Autowired
    private IRuleGroupService ruleGroupService;
    @Autowired
    private IRuleBaseService ruleBaseService;
    @Autowired
    private IRuleAccountThresholdService ruleAccountThresholdService;
    @Autowired
    private IRuleAccountTemplateService accountTemplateService;


    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "规则模板-通过id查询")
    @ApiOperation(value = "规则模板-通过id查询", notes = "规则模板-通过id查询")
    @GetMapping(value = "/queryDetailById")
    public Result<JSONObject> queryDetailById(@RequestParam(name = "id", required = true) String id) {
        Result<JSONObject> result = new Result<>();
        try {
            JSONObject ruleTemplateJson = ruleTemplateService.queryDetailById(id);
            result.setSuccess(true);
            result.setResult(ruleTemplateJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }

        return result;
    }



    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "规则模板-通过id查询")
    @ApiOperation(value = "规则模板-通过id查询", notes = "规则模板-通过id查询")
    @GetMapping(value = "/previewTemplate")
    public Result<JSONObject> previewTemplate(@RequestParam(name = "id", required = true) String id) {
        Result<JSONObject> result = new Result<>();
        try {
            JSONObject ruleTemplateJson = ruleTemplateService.previewTemplate(id);
            result.setSuccess(true);
            result.setResult(ruleTemplateJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }

        return result;
    }

    /**
     * 根据 模板id  账户id 查询详情
     *
     * @param
     * @return
     */

    @GetMapping(value = "/queryDetailByAccountIdAndTemplateId")
    public Result<JSONObject> queryDetailByAccountIdAndTemplateId(Long accountId, Long templateId) {
        Result<JSONObject> result = new Result<>();
        try {
            JSONObject ruleTemplateJson = ruleTemplateService.queryDetailByAccountIdAndTemplateId(accountId, templateId);
            result.setSuccess(true);
            result.setResult(ruleTemplateJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }

        return result;
    }


    /**
     * 分页列表查询
     *
     * @param ruleTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "规则模板-分页列表查询")
    @ApiOperation(value = "规则模板-分页列表查询", notes = "规则模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<RuleTemplate>> queryPageList(RuleTemplate ruleTemplate,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {
        Result<IPage<RuleTemplate>> result = new Result<>();
        String templateName = ruleTemplate.getTemplateName();
        ruleTemplate.setTemplateName(null);
        QueryWrapper<RuleTemplate> queryWrapper = QueryGenerator.initQueryWrapper(ruleTemplate, req.getParameterMap());
        if (!Check.isNull(templateName)) {
            queryWrapper.like("template_name", templateName);
        }
        Page<RuleTemplate> page = new Page<RuleTemplate>(pageNo, pageSize);
        IPage<RuleTemplate> pageList = ruleTemplateService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    /**
     * 添加
     *
     * @param requestJson
     * @return
     */
    @AutoLog(value = "创建规则模板")

    @PostMapping(value = "/createTemplate")
    public Result createTemplate(@RequestBody JSONObject requestJson) {
        Result result = new Result<>();
        try {
            if (Check.isNull(requestJson)) {
                throw new Exception("请传入模板入参");
            }
            JSONObject returnJson = ruleTemplateService.createTemplate(requestJson);
            if (returnJson.getInteger("code") == 0) {
                result.setSuccess(true);
                result.setMessage(returnJson.getString("message"));
            } else {
                result.setSuccess(false);
                result.setMessage(returnJson.getString("message"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    /**
     * 添加
     *
     * @param ruleTemplate
     * @return
     */
    @AutoLog(value = "规则模板-添加")
    @ApiOperation(value = "规则模板-添加", notes = "规则模板-添加")
    @PostMapping(value = "/add")
    public Result<RuleTemplate> add(@RequestBody RuleTemplate ruleTemplate) {
        Result<RuleTemplate> result = new Result<>();
        try {
            ruleTemplateService.save(ruleTemplate);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param ruleTemplate
     * @return
     */
    @AutoLog(value = "规则模板-编辑")
    @ApiOperation(value = "规则模板-编辑", notes = "规则模板-编辑")
    @PutMapping(value = "/edit")
    public Result<RuleTemplate> edit(@RequestBody RuleTemplate ruleTemplate) {
        Result<RuleTemplate> result = new Result<RuleTemplate>();
        RuleTemplate ruleTemplateEntity = ruleTemplateService.getById(ruleTemplate.getId());
        if (ruleTemplateEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = ruleTemplateService.updateById(ruleTemplate);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */


    @AutoLog(value = "规则模板-通过id删除")
    @ApiOperation(value = "规则模板-通过id删除", notes = "规则模板-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            RuleTemplate ruleTemplate = ruleTemplateService.getById(id);
            if (!Check.isNull(ruleTemplate)) {
                QueryWrapper<RuleGroup> groupQueryWrapper = new QueryWrapper<>();
                groupQueryWrapper.eq("template_id", id);
                List<RuleGroup> groupList = ruleGroupService.list(groupQueryWrapper);
                if (!Check.isNull(groupList)) {
                    for (RuleGroup group : groupList) {
                        JSONArray ruleIds = JSONArray.parseArray(group.getRuleIds());
                        if (Check.isNull(ruleIds)) {
                            continue;
                        }
                        for (int i = 0; i < ruleIds.size(); i++) {
                            Long ruleId = ruleIds.getLong(i);
                            ruleBaseService.removeById(ruleId);
                            Map<String, Object> deleteMap = new HashMap<>();
                            deleteMap.put("rule_id", ruleId);
                            ruleAccountThresholdService.removeByMap(deleteMap);
                        }
                        ruleGroupService.removeById(group.getId());
                    }
                }

                Map<String, Object> deleteMap = new HashMap<>();
                deleteMap.put("template_id", id);
                accountTemplateService.removeByMap(deleteMap);
                ruleTemplateService.removeById(id);
            }


        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "规则模板-批量删除")
    @ApiOperation(value = "规则模板-批量删除", notes = "规则模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<RuleTemplate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<RuleTemplate> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.ruleTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "规则模板-通过id查询")
    @ApiOperation(value = "规则模板-通过id查询", notes = "规则模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<RuleTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<RuleTemplate> result = new Result<>();
        RuleTemplate ruleTemplate = ruleTemplateService.getById(id);
        if (ruleTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(ruleTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<RuleTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                RuleTemplate ruleTemplate = JSON.parseObject(deString, RuleTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(ruleTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<RuleTemplate> pageList = ruleTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "规则模板列表");
        mv.addObject(NormalExcelConstants.CLASS, RuleTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("规则模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<RuleTemplate> listRuleTemplates = ExcelImportUtil.importExcel(file.getInputStream(), RuleTemplate.class, params);
                ruleTemplateService.saveBatch(listRuleTemplates);
                return Result.ok("文件导入成功！数据行数:" + listRuleTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
