package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.CollectionSlogans;
import cn.com.ctop.kuaishou.modules.batch.service.ICollectionSlogansService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 收藏的广告语
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-10-12
 */
@Slf4j
@Api(tags = "收藏的广告语")
@RestController
@RequestMapping("/kuaishou/collectionSlogans")
public class CollectionSlogansController {
    @Autowired
    private ICollectionSlogansService CollectionSlogansService;

    /**
     * 分页列表查询
     *
     * @param CollectionSlogans
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "收藏的广告语-分页列表查询")
    @ApiOperation(value = "收藏的广告语-分页列表查询", notes = "收藏的广告语-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<CollectionSlogans>> queryPageList(CollectionSlogans CollectionSlogans,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<cn.com.ctop.kuaishou.modules.batch.entity.CollectionSlogans>> result = new Result<>();
        QueryWrapper<CollectionSlogans> queryWrapper = QueryGenerator.initQueryWrapper(CollectionSlogans, req.getParameterMap());
        Page<CollectionSlogans> page = new Page<CollectionSlogans>(pageNo, pageSize);
        IPage<CollectionSlogans> pageList = CollectionSlogansService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param CollectionSlogans
     * @return
     */
    @AutoLog(value = "收藏的广告语-添加")
    @ApiOperation(value = "收藏的广告语-添加", notes = "收藏的广告语-添加")
    @PostMapping(value = "/add")
    public Result<JSONObject> add(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {
            if(Check.isNull(requestJson.getJSONArray("coverSlogansList"))&&Check.isNull(requestJson.getJSONArray("captionsList"))&&Check.isNull(requestJson.getJSONArray("creativeTag"))){
                result.setMessage("请输入广告语");
                result.setSuccess(false);
                return result;
            }
            CollectionSlogansService.saveCollectionSlogans(requestJson);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param CollectionSlogans
     * @return
     */
    @AutoLog(value = "收藏的广告语-编辑")
    @ApiOperation(value = "收藏的广告语-编辑", notes = "收藏的广告语-编辑")
    @PutMapping(value = "/edit")
    public Result<CollectionSlogans> edit(@RequestBody CollectionSlogans CollectionSlogans) {
        Result<CollectionSlogans> result = new Result<CollectionSlogans>();
        CollectionSlogans CollectionSlogansEntity = CollectionSlogansService.getById(CollectionSlogans.getId());
        if (CollectionSlogansEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = CollectionSlogansService.updateById(CollectionSlogans);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "收藏的广告语-通过id删除")
    @ApiOperation(value = "收藏的广告语-通过id删除", notes = "收藏的广告语-通过id删除")
    @GetMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            CollectionSlogansService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "收藏的广告语-批量删除")
    @ApiOperation(value = "收藏的广告语-批量删除", notes = "收藏的广告语-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<CollectionSlogans> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<CollectionSlogans> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.CollectionSlogansService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "收藏的广告语-通过userId查询")
    @ApiOperation(value = "收藏的广告语-通过id查询", notes = "收藏的广告语-通过id查询")
    @GetMapping(value = "/getByUserId")
    public Result<JSONObject> queryById(String userId,String accountId,String type) {
        Result<JSONObject> result = new Result<>();
        JSONObject CollectionSlogans = CollectionSlogansService.getByUserId(userId,accountId,type);
        if (CollectionSlogans == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(CollectionSlogans);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<CollectionSlogans> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                CollectionSlogans CollectionSlogans = JSON.parseObject(deString, CollectionSlogans.class);
                queryWrapper = QueryGenerator.initQueryWrapper(CollectionSlogans, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<CollectionSlogans> pageList = CollectionSlogansService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "收藏的广告语列表");
        mv.addObject(NormalExcelConstants.CLASS, CollectionSlogans.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("收藏的广告语列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<CollectionSlogans> listCollectionSloganss = ExcelImportUtil.importExcel(file.getInputStream(), CollectionSlogans.class, params);
                CollectionSlogansService.saveBatch(listCollectionSloganss);
                return Result.ok("文件导入成功！数据行数:" + listCollectionSloganss.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
