package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouAppInfo;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouAppInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 广告组-app信息
 * @Author: jeecg-boot
 * @Date: 2019-07-23
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "广告组-app信息")
@RestController
@RequestMapping("/kuaishou/kuaiShouAppInfo")
public class KuaiShouAppInfoController {
    @Autowired
    private IKuaiShouAppInfoService kuaiShouAppInfoService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouAppInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "广告组-app信息-分页列表查询")
    @ApiOperation(value = "广告组-app信息-分页列表查询", notes = "广告组-app信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouAppInfo>> queryPageList(KuaiShouAppInfo kuaiShouAppInfo,
                                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                        HttpServletRequest req) {
        Result<IPage<KuaiShouAppInfo>> result = new Result<>();
        QueryWrapper<KuaiShouAppInfo> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouAppInfo, req.getParameterMap());
        Page<KuaiShouAppInfo> page = new Page<>(pageNo, pageSize);
        IPage<KuaiShouAppInfo> pageList = kuaiShouAppInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouAppInfo
     * @return
     */
    @AutoLog(value = "广告组-app信息-添加")
    @ApiOperation(value = "广告组-app信息-添加", notes = "广告组-app信息-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouAppInfo> add(@RequestBody KuaiShouAppInfo kuaiShouAppInfo) {
        Result<KuaiShouAppInfo> result = new Result<>();
        try {
            kuaiShouAppInfoService.save(kuaiShouAppInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouAppInfo
     * @return
     */
    @AutoLog(value = "广告组-app信息-编辑")
    @ApiOperation(value = "广告组-app信息-编辑", notes = "广告组-app信息-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouAppInfo> edit(@RequestBody KuaiShouAppInfo kuaiShouAppInfo) {
        Result<KuaiShouAppInfo> result = new Result<>();
        KuaiShouAppInfo kuaiShouAppInfoEntity = kuaiShouAppInfoService.getById(kuaiShouAppInfo.getId());
        if (kuaiShouAppInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouAppInfoService.updateById(kuaiShouAppInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告组-app信息-通过id删除")
    @ApiOperation(value = "广告组-app信息-通过id删除", notes = "广告组-app信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouAppInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告组-app信息-批量删除")
    @ApiOperation(value = "广告组-app信息-批量删除", notes = "广告组-app信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouAppInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouAppInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouAppInfoService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告组-app信息-通过id查询")
    @ApiOperation(value = "广告组-app信息-通过id查询", notes = "广告组-app信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouAppInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouAppInfo> result = new Result<>();
        KuaiShouAppInfo kuaiShouAppInfo = kuaiShouAppInfoService.getById(id);
        if (kuaiShouAppInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouAppInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
