package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouTrackUrlCollection;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouTrackUrlCollectionService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 监测链接收藏
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-07-22
 */
@Slf4j
@Api(tags = "监测链接收藏")
@RestController
@RequestMapping("/kuaishou/kuaiShouTrackUrlCollection")
public class KuaiShouTrackUrlCollectionController {
    @Autowired
    private IKuaiShouTrackUrlCollectionService kuaiShouTrackUrlCollectionService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouTrackUrlCollection
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "监测链接收藏-分页列表查询")
    @ApiOperation(value = "监测链接收藏-分页列表查询", notes = "监测链接收藏-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouTrackUrlCollection>> queryPageList(KuaiShouTrackUrlCollection kuaiShouTrackUrlCollection,
                                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                   HttpServletRequest req) {
        Result<IPage<KuaiShouTrackUrlCollection>> result = new Result<>();
        String trackName = kuaiShouTrackUrlCollection.getTrackName();
        kuaiShouTrackUrlCollection.setTrackName(null);


        QueryWrapper<KuaiShouTrackUrlCollection> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouTrackUrlCollection, req.getParameterMap());
        if (!Check.isNull(trackName)) {
            queryWrapper.like("track_name", trackName);
        }

        Page<KuaiShouTrackUrlCollection> page = new Page<KuaiShouTrackUrlCollection>(pageNo, pageSize);
        IPage<KuaiShouTrackUrlCollection> pageList = kuaiShouTrackUrlCollectionService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    @GetMapping(value = "/checkTrackName")
    public Result<Boolean> checkTrackName(Long accountId, String trackName, String userId) {
        Result<Boolean> result = new Result<>();
        QueryWrapper<KuaiShouTrackUrlCollection> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        queryWrapper.eq("track_name", trackName);
        queryWrapper.eq("user_id", userId);
        List<KuaiShouTrackUrlCollection> list = kuaiShouTrackUrlCollectionService.list(queryWrapper);
        if (Check.isNull(list)) {
            result.setResult(true);
        } else {
            result.setResult(false);
        }

        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouTrackUrlCollection
     * @return
     */
    @AutoLog(value = "监测链接收藏-添加")
    @ApiOperation(value = "监测链接收藏-添加", notes = "监测链接收藏-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouTrackUrlCollection> add(@RequestBody KuaiShouTrackUrlCollection kuaiShouTrackUrlCollection) {
        Result<KuaiShouTrackUrlCollection> result = new Result<>();
        try {
            kuaiShouTrackUrlCollectionService.save(kuaiShouTrackUrlCollection);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouTrackUrlCollection
     * @return
     */
    @AutoLog(value = "监测链接收藏-编辑")
    @ApiOperation(value = "监测链接收藏-编辑", notes = "监测链接收藏-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouTrackUrlCollection> edit(@RequestBody KuaiShouTrackUrlCollection kuaiShouTrackUrlCollection) {
        Result<KuaiShouTrackUrlCollection> result = new Result<KuaiShouTrackUrlCollection>();
        KuaiShouTrackUrlCollection kuaiShouTrackUrlCollectionEntity = kuaiShouTrackUrlCollectionService.getById(kuaiShouTrackUrlCollection.getId());
        if (kuaiShouTrackUrlCollectionEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouTrackUrlCollectionService.updateById(kuaiShouTrackUrlCollection);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "监测链接收藏-通过id删除")
    @ApiOperation(value = "监测链接收藏-通过id删除", notes = "监测链接收藏-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouTrackUrlCollectionService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "监测链接收藏-批量删除")
    @ApiOperation(value = "监测链接收藏-批量删除", notes = "监测链接收藏-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouTrackUrlCollection> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouTrackUrlCollection> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouTrackUrlCollectionService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "监测链接收藏-通过id查询")
    @ApiOperation(value = "监测链接收藏-通过id查询", notes = "监测链接收藏-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouTrackUrlCollection> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouTrackUrlCollection> result = new Result<>();
        KuaiShouTrackUrlCollection kuaiShouTrackUrlCollection = kuaiShouTrackUrlCollectionService.getById(id);
        if (kuaiShouTrackUrlCollection == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouTrackUrlCollection);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouTrackUrlCollection> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouTrackUrlCollection kuaiShouTrackUrlCollection = JSON.parseObject(deString, KuaiShouTrackUrlCollection.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouTrackUrlCollection, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouTrackUrlCollection> pageList = kuaiShouTrackUrlCollectionService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "监测链接收藏列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouTrackUrlCollection.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("监测链接收藏列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouTrackUrlCollection> listKuaiShouTrackUrlCollections = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouTrackUrlCollection.class, params);
                kuaiShouTrackUrlCollectionService.saveBatch(listKuaiShouTrackUrlCollections);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouTrackUrlCollections.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
