package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouProgramCreative;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouProgramCreativeService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 快手-程序化创意
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-09-22
 */
@Slf4j
@Api(tags = "快手-程序化创意")
@RestController
@RequestMapping("/ctop/kuaishouProgramCreative")
public class KuaishouProgramCreativeController {
    @Autowired
    private IKuaishouProgramCreativeService kuaishouProgramCreativeService;
    @Autowired
    private ICtopOauthTokenService oauthTokenService;

    /**
     * 分页列表查询
     *
     * @param kuaishouProgramCreative
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手-程序化创意-分页列表查询")
    @ApiOperation(value = "快手-程序化创意-分页列表查询", notes = "快手-程序化创意-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouProgramCreative>> queryPageList(KuaishouProgramCreative kuaishouProgramCreative,
                                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                HttpServletRequest req) {
        Result<IPage<KuaishouProgramCreative>> result = new Result<>();
        try {
            if (Check.isNull(kuaishouProgramCreative.getAccountId()) || Check.isNull(kuaishouProgramCreative.getUnitId())) {
                throw new Exception("必填参数不能为空");
            }
            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(kuaishouProgramCreative.getAccountId());
            if (Check.isNull(oauthToken)) {
                throw new Exception("未获取到账户信息");
            }
            kuaishouProgramCreativeService.getProgramCreative(oauthToken.getAccountId(), kuaishouProgramCreative.getUnitId(), oauthToken.getAccessToken(), null, null, 1);
            QueryWrapper<KuaishouProgramCreative> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouProgramCreative, req.getParameterMap());
            Page<KuaishouProgramCreative> page = new Page<KuaishouProgramCreative>(pageNo, pageSize);
            IPage<KuaishouProgramCreative> pageList = kuaishouProgramCreativeService.page(page, queryWrapper);
            result.setSuccess(true);
            result.setResult(pageList);
        } catch (Exception e) {
            result.setMessage(e.getMessage());
        }

        return result;
    }


    /**
     * 创建程序化创意
     *
     * @param requestJson
     * @return
     */

    @PostMapping(value = "/createProgramCreative")
    public Result<JSONObject> createProgramCreative(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {
            if (Check.isNull(requestJson)) {
                throw new Exception("创建程序化创意参数不能为空");
            }

            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                throw new Exception("请选择账户id");
            }
            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                throw new Exception("未获取到账户信息");
            }


            JSONObject returnJson = kuaishouProgramCreativeService.createProgramCreative(oauthToken, requestJson);
            if (returnJson.getBoolean("isTrueOrFalse")) {
                result.setSuccess(true);
                result.setMessage("创建成功");
            } else {
                result.setSuccess(false);
                result.setMessage(returnJson.getString("message"));
            }

        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    /**
     * 创意分类
     *
     * @param
     * @return
     */
    @PostMapping(value = "/getProgramCreativeCover")
    public Result<JSONArray> getCreativeCategory(@RequestBody JSONObject jsonObject) {
        Result<JSONArray> result = new Result<>();
        try {

             JSONArray md5Arr = jsonObject.getJSONArray("md5Arr");

            if (Check.isNull(md5Arr)) {
                throw new Exception("请选择视频");
            }

            JSONArray data = kuaishouProgramCreativeService.getProgramCreativeCover(md5Arr);
            result.setSuccess(true);
            result.setResult(data);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setResult(null);
            result.setMessage(e.getMessage());
        }

        return result;
    }


    /**
     * 添加
     *
     * @param kuaishouProgramCreative
     * @return
     */
    @AutoLog(value = "快手-程序化创意-添加")
    @ApiOperation(value = "快手-程序化创意-添加", notes = "快手-程序化创意-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouProgramCreative> add(@RequestBody KuaishouProgramCreative kuaishouProgramCreative) {
        Result<KuaishouProgramCreative> result = new Result<>();
        try {
            kuaishouProgramCreativeService.save(kuaishouProgramCreative);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouProgramCreative
     * @return
     */
    @AutoLog(value = "快手-程序化创意-编辑")
    @ApiOperation(value = "快手-程序化创意-编辑", notes = "快手-程序化创意-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouProgramCreative> edit(@RequestBody KuaishouProgramCreative kuaishouProgramCreative) {
        Result<KuaishouProgramCreative> result = new Result<KuaishouProgramCreative>();
        KuaishouProgramCreative kuaishouProgramCreativeEntity = kuaishouProgramCreativeService.getById(kuaishouProgramCreative.getId());
        if (kuaishouProgramCreativeEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouProgramCreativeService.updateById(kuaishouProgramCreative);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-程序化创意-通过id删除")
    @ApiOperation(value = "快手-程序化创意-通过id删除", notes = "快手-程序化创意-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouProgramCreativeService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-程序化创意-批量删除")
    @ApiOperation(value = "快手-程序化创意-批量删除", notes = "快手-程序化创意-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouProgramCreative> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouProgramCreative> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouProgramCreativeService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-程序化创意-通过id查询")
    @ApiOperation(value = "快手-程序化创意-通过id查询", notes = "快手-程序化创意-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouProgramCreative> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouProgramCreative> result = new Result<>();
        KuaishouProgramCreative kuaishouProgramCreative = kuaishouProgramCreativeService.getById(id);
        if (kuaishouProgramCreative == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouProgramCreative);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouProgramCreative> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouProgramCreative kuaishouProgramCreative = JSON.parseObject(deString, KuaishouProgramCreative.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouProgramCreative, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouProgramCreative> pageList = kuaishouProgramCreativeService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手-程序化创意列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouProgramCreative.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-程序化创意列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouProgramCreative> listKuaishouProgramCreatives = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouProgramCreative.class, params);
                kuaishouProgramCreativeService.saveBatch(listKuaishouProgramCreatives);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouProgramCreatives.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
