package cn.com.ctop.kuaishou.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.KuaiShouCleanMaterialReport;
import cn.com.ctop.common.module.service.IKuaiShouCleanMaterialReportService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 快手素材清洗数据
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-02-24
 */
@Slf4j
@Api(tags = "快手素材清洗数据")
@RestController
@RequestMapping("/ctop/kuaiShouCleanMaterialReport")
public class KuaiShouCleanMaterialReportController {
    @Autowired
    private IKuaiShouCleanMaterialReportService kuaiShouCleanMaterialReportService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouCleanMaterialReport
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手素材清洗数据-分页列表查询")
    @ApiOperation(value = "快手素材清洗数据-分页列表查询", notes = "快手素材清洗数据-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouCleanMaterialReport>> queryPageList(KuaiShouCleanMaterialReport kuaiShouCleanMaterialReport,
                                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                    HttpServletRequest req) {
        Result<IPage<KuaiShouCleanMaterialReport>> result = new Result<>();
        QueryWrapper<KuaiShouCleanMaterialReport> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouCleanMaterialReport, req.getParameterMap());
        Page<KuaiShouCleanMaterialReport> page = new Page<>(pageNo, pageSize);
        IPage<KuaiShouCleanMaterialReport> pageList = kuaiShouCleanMaterialReportService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouCleanMaterialReport
     * @return
     */
    @AutoLog(value = "快手素材清洗数据-添加")
    @ApiOperation(value = "快手素材清洗数据-添加", notes = "快手素材清洗数据-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouCleanMaterialReport> add(@RequestBody KuaiShouCleanMaterialReport kuaiShouCleanMaterialReport) {
        Result<KuaiShouCleanMaterialReport> result = new Result<>();
        try {
            kuaiShouCleanMaterialReportService.save(kuaiShouCleanMaterialReport);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouCleanMaterialReport
     * @return
     */
    @AutoLog(value = "快手素材清洗数据-编辑")
    @ApiOperation(value = "快手素材清洗数据-编辑", notes = "快手素材清洗数据-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouCleanMaterialReport> edit(@RequestBody KuaiShouCleanMaterialReport kuaiShouCleanMaterialReport) {
        Result<KuaiShouCleanMaterialReport> result = new Result<>();
        KuaiShouCleanMaterialReport kuaiShouCleanMaterialReportEntity = kuaiShouCleanMaterialReportService.getById(kuaiShouCleanMaterialReport.getId());
        if (kuaiShouCleanMaterialReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouCleanMaterialReportService.updateById(kuaiShouCleanMaterialReport);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手素材清洗数据-通过id删除")
    @ApiOperation(value = "快手素材清洗数据-通过id删除", notes = "快手素材清洗数据-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouCleanMaterialReportService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手素材清洗数据-批量删除")
    @ApiOperation(value = "快手素材清洗数据-批量删除", notes = "快手素材清洗数据-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouCleanMaterialReport> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouCleanMaterialReport> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouCleanMaterialReportService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手素材清洗数据-通过id查询")
    @ApiOperation(value = "快手素材清洗数据-通过id查询", notes = "快手素材清洗数据-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouCleanMaterialReport> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouCleanMaterialReport> result = new Result<>();
        KuaiShouCleanMaterialReport kuaiShouCleanMaterialReport = kuaiShouCleanMaterialReportService.getById(id);
        if (kuaiShouCleanMaterialReport == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouCleanMaterialReport);
            result.setSuccess(true);
        }
        return result;
    }
}
