package cn.com.ctop.oa.modules.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.oa.modules.entity.WechatCheckinData;
import cn.com.ctop.oa.modules.entity.WechatCheckinSubData;
import cn.com.ctop.oa.modules.service.IWechatCheckinDataService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 企业微信用户
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-09-15
 */
@Slf4j
@Api(tags = "企业微信用户")
@RestController
@RequestMapping("/oa/wechatCheckinData")
public class WechatCheckinDataController {
    @Autowired
    private IWechatCheckinDataService wechatCheckinDataService;

    /**
     * 分页列表查询
     *
     * @param wechatCheckinData
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "考勤打卡明细-分页列表查询")
    @ApiOperation(value = "考勤打卡明细-分页列表查询", notes = "考勤打卡明细-分页列表查询")
    @PostMapping(value = "/queryPageList")
    public Result<PageInfo<WechatCheckinSubData>> queryPageList(@RequestBody Map<String, Object> paramsMap) {
        Result<PageInfo<WechatCheckinSubData>> result = new Result<>();
        try {
            PageInfo<WechatCheckinSubData> pageData = wechatCheckinDataService.selectPageList(paramsMap);
            if (!Check.isNull(pageData)) {
                result.setResult(pageData);
                result.setSuccess(true);
            }
        } catch (Exception e) {
            log.error("考勤打卡明细-分页列表查询 error ", e);
            result.error500("操作失败");
        }
        return result;
    }


    /**
     * 添加
     *
     * @param wechatCheckinData
     * @return
     */
    @AutoLog(value = "企业微信用户-添加")
    @ApiOperation(value = "企业微信用户-添加", notes = "企业微信用户-添加")
    @PostMapping(value = "/add")
    public Result<WechatCheckinData> add(@RequestBody WechatCheckinData wechatCheckinData) {
        Result<WechatCheckinData> result = new Result<>();
        try {
            wechatCheckinDataService.save(wechatCheckinData);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param wechatCheckinData
     * @return
     */
    @AutoLog(value = "企业微信用户-编辑")
    @ApiOperation(value = "企业微信用户-编辑", notes = "企业微信用户-编辑")
    @PutMapping(value = "/edit")
    public Result<WechatCheckinData> edit(@RequestBody WechatCheckinData wechatCheckinData) {
        Result<WechatCheckinData> result = new Result<WechatCheckinData>();
        WechatCheckinData wechatCheckinDataEntity = wechatCheckinDataService.getById(wechatCheckinData.getId());
        if (wechatCheckinDataEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = wechatCheckinDataService.updateById(wechatCheckinData);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "企业微信用户-通过id删除")
    @ApiOperation(value = "企业微信用户-通过id删除", notes = "企业微信用户-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            wechatCheckinDataService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "企业微信用户-批量删除")
    @ApiOperation(value = "企业微信用户-批量删除", notes = "企业微信用户-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<WechatCheckinData> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<WechatCheckinData> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.wechatCheckinDataService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "企业微信用户-通过id查询")
    @ApiOperation(value = "企业微信用户-通过id查询", notes = "企业微信用户-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<WechatCheckinData> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<WechatCheckinData> result = new Result<>();
        WechatCheckinData wechatCheckinData = wechatCheckinDataService.getById(id);
        if (wechatCheckinData == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(wechatCheckinData);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<WechatCheckinData> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                WechatCheckinData wechatCheckinData = JSON.parseObject(deString, WechatCheckinData.class);
                queryWrapper = QueryGenerator.initQueryWrapper(wechatCheckinData, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<WechatCheckinData> pageList = wechatCheckinDataService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "企业微信用户列表");
        mv.addObject(NormalExcelConstants.CLASS, WechatCheckinData.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("企业微信用户列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<WechatCheckinData> listWechatCheckinDatas = ExcelImportUtil.importExcel(file.getInputStream(), WechatCheckinData.class, params);
                wechatCheckinDataService.saveBatch(listWechatCheckinDatas);
                return Result.ok("文件导入成功！数据行数:" + listWechatCheckinDatas.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
