package cn.com.ctop.oa.modules.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.ExportExcelUtils;
import cn.com.ctop.common.module.utils.HttpUtils;
import cn.com.ctop.oa.modules.entity.WechatAttendance;
import cn.com.ctop.oa.modules.entity.WechatNoList;
import cn.com.ctop.oa.modules.service.IWechatNoListService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 审批信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-09-23
 */
@Slf4j
@Api(tags = "审批信息")
@RestController
@RequestMapping("/oa/wechatNoList")
public class WechatNoListController {
    @Autowired
    private IWechatNoListService wechatNoListService;

    /**
     * 分页列表查询
     *
     * @param wechatNoList
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "审批信息-分页列表查询")
    @ApiOperation(value = "审批信息-分页列表查询", notes = "审批信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<WechatNoList>> queryPageList(WechatNoList wechatNoList,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {
        Result<IPage<WechatNoList>> result = new Result<>();
        QueryWrapper<WechatNoList> queryWrapper = QueryGenerator.initQueryWrapper(wechatNoList, req.getParameterMap());
        Page<WechatNoList> page = new Page<WechatNoList>(pageNo, pageSize);
        IPage<WechatNoList> pageList = wechatNoListService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param wechatNoList
     * @return
     */
    @AutoLog(value = "审批信息-添加")
    @ApiOperation(value = "审批信息-添加", notes = "审批信息-添加")
    @PostMapping(value = "/add")
    public Result<WechatNoList> add(@RequestBody WechatNoList wechatNoList) {
        Result<WechatNoList> result = new Result<>();
        try {
            wechatNoListService.save(wechatNoList);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param wechatNoList
     * @return
     */
    @AutoLog(value = "审批信息-编辑")
    @ApiOperation(value = "审批信息-编辑", notes = "审批信息-编辑")
    @PutMapping(value = "/edit")
    public Result<WechatNoList> edit(@RequestBody WechatNoList wechatNoList) {
        Result<WechatNoList> result = new Result<WechatNoList>();
        WechatNoList wechatNoListEntity = wechatNoListService.getById(wechatNoList.getId());
        if (wechatNoListEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = wechatNoListService.updateById(wechatNoList);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "审批信息-通过id删除")
    @ApiOperation(value = "审批信息-通过id删除", notes = "审批信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            wechatNoListService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "审批信息-批量删除")
    @ApiOperation(value = "审批信息-批量删除", notes = "审批信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<WechatNoList> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<WechatNoList> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.wechatNoListService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "审批信息-通过id查询")
    @ApiOperation(value = "审批信息-通过id查询", notes = "审批信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<WechatNoList> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<WechatNoList> result = new Result<>();
        WechatNoList wechatNoList = wechatNoListService.getById(id);
        if (wechatNoList == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(wechatNoList);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<WechatNoList> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                WechatNoList wechatNoList = JSON.parseObject(deString, WechatNoList.class);
                queryWrapper = QueryGenerator.initQueryWrapper(wechatNoList, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<WechatNoList> pageList = wechatNoListService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "审批信息列表");
        mv.addObject(NormalExcelConstants.CLASS, WechatNoList.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("审批信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return AbnormalAttendance
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<WechatNoList> listWechatNoLists = ExcelImportUtil.importExcel(file.getInputStream(), WechatNoList.class, params);
                wechatNoListService.saveBatch(listWechatNoLists);
                return Result.ok("文件导入成功！数据行数:" + listWechatNoLists.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /**
     * 考勤异常记录 导出数据
     *
     * @param response
     */
    @GetMapping("/excel")
    public void getExcelReport(HttpServletResponse response, String startTime, String stopTime, String departName, Long departId, String userName, String userId) {
        Map<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("startTime", startTime);
        paramsMap.put("stopTime", stopTime);
        paramsMap.put("departName", departName);
        paramsMap.put("departId", departId);
        paramsMap.put("userName", userName);
        paramsMap.put("userId", userId);
        List<WechatAttendance> excelData = wechatNoListService.queryAttendanceData(paramsMap);
        List<WechatAttendance> excelYRData = wechatNoListService.queryYRAttendanceData(paramsMap);
        List<WechatAttendance> overtimeData = wechatNoListService.queryOverTimeData(paramsMap);
        List<WechatAttendance> overtimeYRData = wechatNoListService.queryYROverTimeData(paramsMap);
        try {
            List<List<Object>> excelList = new ArrayList<>();
            List<List<Object>> excelYRList = new ArrayList<>();
            List<List<Object>> overtimeExcelList = new ArrayList<>();
            List<List<Object>> overtimeYRExcelList = new ArrayList<>();
            if (!excelData.isEmpty()) {
                excelData.forEach(data -> {
                    List<Object> temp = new ArrayList<>();
                    temp.add(data.getUserName());
                    temp.add(data.getDepartName());
                    temp.add(data.getCheckinDate());
                    temp.add(data.getCheckinType());
                    temp.add(data.getExceptionType());
                    temp.add(data.getCheckinTime());
                    temp.add(data.getType());
                    temp.add(data.getStateDuration());
                    if (!Check.isNull(data.getStatus())) {
                        temp.add("1".equals(data.getStatus()) ? "审批中" : "已通过");
                    } else {
                        temp.add("");
                    }
                    excelList.add(temp);
                });
            }
            if (!overtimeData.isEmpty()) {
                overtimeData.forEach(data -> {
                    List<Object> temp = new ArrayList<>();
                    temp.add(data.getUserName());
                    temp.add(data.getDepartName());
                    temp.add(data.getCheckinDate());
                    temp.add(data.getOvertime());
                    overtimeExcelList.add(temp);
                });
            }
            if (!excelYRData.isEmpty()) {
                excelYRData.forEach(data -> {
                    List<Object> temp = new ArrayList<>();
                    temp.add(data.getUserName());
                    temp.add(data.getDepartName());
                    temp.add(data.getCheckinDate());
                    temp.add(data.getCheckinType());
                    temp.add(data.getExceptionType());
                    temp.add(data.getCheckinTime());
                    temp.add(data.getType());
                    temp.add(data.getStateDuration());
                    if (!Check.isNull(data.getStatus())) {
                        temp.add("1".equals(data.getStatus()) ? "审批中" : "已通过");
                    } else {
                        temp.add("");
                    }
                    excelYRList.add(temp);
                });
            }
            if (!overtimeYRData.isEmpty()) {
                overtimeYRData.forEach(data -> {
                    List<Object> temp = new ArrayList<>();
                    temp.add(data.getUserName());
                    temp.add(data.getDepartName());
                    temp.add(data.getCheckinDate());
                    temp.add(data.getOvertime());
                    overtimeYRExcelList.add(temp);
                });
            }
            OutputStream os = response.getOutputStream();
            ExportExcelUtils eeu = new ExportExcelUtils();
            XSSFWorkbook workbook = new XSSFWorkbook();
            String[] titles = {"姓名", "部门", "考勤日期", "考勤类型", "考勤状态", "考勤时间", "审核状态", "状态时长", "审批状态"};
            String[] overtimeTitles = {"姓名", "部门", "加班日期", "加班时长"};
            eeu.exportExcelForAttendance(workbook, 0, "汇创异常考勤", titles, excelList);
            eeu.exportExcel(workbook, 1, "汇创加班考勤", overtimeTitles, overtimeExcelList);
            eeu.exportExcelForAttendance(workbook, 2, "优容异常考勤", titles, excelYRList);
            eeu.exportExcel(workbook, 3, "优容加班考勤", overtimeTitles, overtimeYRExcelList);
            HttpUtils.setResponseHeader(response, "clockIn.xlsx");
            workbook.write(os);
            os.flush();
            os.close();
        } catch (IOException e) {
            log.error(e.getMessage());
        }
    }

    /**
     * 考勤异常数据查询（分页）
     *
     * @param id
     * @return
     */
    @AutoLog(value = "异常考勤数据-分页列表查询")
    @PostMapping(value = "/selectNoListPage")
    public Result<PageInfo<WechatAttendance>> selectNoListPage(@RequestBody Map<String, Object> paramsMap) {
        Result<PageInfo<WechatAttendance>> result = new Result<>();
        try {
            PageInfo<WechatAttendance> pageData = wechatNoListService.selectNoListPage(paramsMap);
            if (!Check.isNull(pageData)) {
                result.setResult(pageData);
                result.setSuccess(true);
            }
        } catch (Exception e) {
            log.error("异常考勤数据-分页列表查询 error ", e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 加班考勤数据查询（分页）
     *
     * @param id
     * @return
     */
    @AutoLog(value = "加班考勤数据-分页列表查询")
    @PostMapping(value = "/selectOverTimePage")
    public Result<PageInfo<WechatAttendance>> selectOverTimePage(@RequestBody Map<String, Object> paramsMap) {
        Result<PageInfo<WechatAttendance>> result = new Result<>();
        try {
            PageInfo<WechatAttendance> pageData = wechatNoListService.selectOverTimePage(paramsMap);
            if (!Check.isNull(pageData)) {
                result.setResult(pageData);
                result.setSuccess(true);
            }
        } catch (Exception e) {
            log.error("加班考勤数据-分页列表查询 error ", e);
            result.error500("操作失败");
        }
        return result;
    }
}
