package cn.com.ctop.shiwan.modules.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.shiwan.modules.entity.ShiwanFileUpload;
import cn.com.ctop.shiwan.modules.service.IShiwanFileUploadService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxl.job.core.enums.NoEn;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 试玩平台，文件信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-10-21
 */
@Slf4j
@Api(tags = "试玩平台，文件信息")
@RestController
@RequestMapping("/shiwan/shiwanFileUpload")
public class ShiwanFileUploadController {
    @Autowired
    private IShiwanFileUploadService shiwanFileUploadService;

    /**
     * 分页列表查询
     *
     * @param shiwanFileUpload
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "试玩平台，文件信息-分页列表查询")
    @ApiOperation(value = "试玩平台，文件信息-分页列表查询", notes = "试玩平台，文件信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ShiwanFileUpload>> queryPageList(ShiwanFileUpload shiwanFileUpload,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<ShiwanFileUpload>> result = new Result<>();
        //父级为0，是zip文件，其他为zip内部文件
        shiwanFileUpload.setParentId((long) 0);
        //状态为1，文件有效未被删除
        shiwanFileUpload.setFileStatus(NoEn.NO1.valueStr());
        QueryWrapper<ShiwanFileUpload> queryWrapper = QueryGenerator.initQueryWrapper(shiwanFileUpload, req.getParameterMap());
        Page<ShiwanFileUpload> page = new Page<ShiwanFileUpload>(pageNo, pageSize);
        IPage<ShiwanFileUpload> pageList = shiwanFileUploadService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @PostMapping("/saveOrUpdateFile")
    public Result<Object> saveOrUpdateFile(@RequestBody MultipartFile file, String userId, Long accountId, String fileType, HttpServletRequest request) {
        Result<Object> result = new Result<>();
        try {
            ShiwanFileUpload entity = shiwanFileUploadService.saveOrUpdateFile(file, userId, accountId, fileType, null, null);
            if (Check.isNull(entity)) {
                result = Result.error("上传失败");
            }
            Object obj = entity;
            result = Result.ok(obj);
        } catch (Exception e) {
            log.error("试玩素材上传COS异常", e);
            result = Result.error(-1, "上传失败");
        }
        return result;
    }


    @PostMapping("/createNewZipFile")
    public Result<Object> createNewZipFile(@RequestBody MultipartFile file, String id, String updateName, String userId, Long accountId, String fileType, String isCreate, String newFileName) {
        Result<Object> result = new Result<>();
        try {
            return shiwanFileUploadService.createNewZipFile(file, id, updateName, userId, accountId, fileType, isCreate, newFileName);
        } catch (Exception e) {
            log.error("生成新的试玩素材ZIP包异常", e);
            result = Result.error(-1, "上传失败");
        }
        return result;
    }

    @GetMapping("/selectListByParentId")
    public Result<List<ShiwanFileUpload>> selectListByParentId(String id) {
        Result<List<ShiwanFileUpload>> result = new Result<>();
        ShiwanFileUpload shiwanFileUploadEntity = shiwanFileUploadService.getById(id);
        if (shiwanFileUploadEntity == null) {
            result.error500("未找到对应实体");
        } else {
            List<ShiwanFileUpload> list = shiwanFileUploadService.selectListByParentId(id);
            if (!Check.isNull(list)) {
                result.success("查询成功!");
                result.setResult(list);
            } else {
                result.error(-1, "查询失败");
            }
        }
        return result;
    }

    /**
     * 添加
     *
     * @param shiwanFileUpload
     * @return
     */
    @AutoLog(value = "试玩平台，文件信息-添加")
    @ApiOperation(value = "试玩平台，文件信息-添加", notes = "试玩平台，文件信息-添加")
    @PostMapping(value = "/add")
    public Result<ShiwanFileUpload> add(@RequestBody ShiwanFileUpload shiwanFileUpload) {
        Result<ShiwanFileUpload> result = new Result<>();
        try {
            shiwanFileUploadService.save(shiwanFileUpload);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param shiwanFileUpload
     * @return
     */
    @AutoLog(value = "试玩平台，文件信息-编辑")
    @ApiOperation(value = "试玩平台，文件信息-编辑", notes = "试玩平台，文件信息-编辑")
    @PostMapping(value = "/edit")
    public Result<ShiwanFileUpload> edit(@RequestBody ShiwanFileUpload shiwanFileUpload) {
        Result<ShiwanFileUpload> result = new Result<ShiwanFileUpload>();
        ShiwanFileUpload shiwanFileUploadEntity = shiwanFileUploadService.getById(shiwanFileUpload.getId());
        if (shiwanFileUploadEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = shiwanFileUploadService.updateById(shiwanFileUpload);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "试玩平台，文件信息-通过id删除")
    @ApiOperation(value = "试玩平台，文件信息-通过id删除", notes = "试玩平台，文件信息-通过id删除")
    @GetMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            shiwanFileUploadService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "试玩平台，文件信息-批量删除")
    @ApiOperation(value = "试玩平台，文件信息-批量删除", notes = "试玩平台，文件信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ShiwanFileUpload> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ShiwanFileUpload> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.shiwanFileUploadService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "试玩平台，文件信息-通过id查询")
    @ApiOperation(value = "试玩平台，文件信息-通过id查询", notes = "试玩平台，文件信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ShiwanFileUpload> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ShiwanFileUpload> result = new Result<>();
        ShiwanFileUpload shiwanFileUpload = shiwanFileUploadService.getById(id);
        if (shiwanFileUpload == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(shiwanFileUpload);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<ShiwanFileUpload> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                ShiwanFileUpload shiwanFileUpload = JSON.parseObject(deString, ShiwanFileUpload.class);
                queryWrapper = QueryGenerator.initQueryWrapper(shiwanFileUpload, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<ShiwanFileUpload> pageList = shiwanFileUploadService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "试玩平台，文件信息列表");
        mv.addObject(NormalExcelConstants.CLASS, ShiwanFileUpload.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("试玩平台，文件信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<ShiwanFileUpload> listShiwanFileUploads = ExcelImportUtil.importExcel(file.getInputStream(), ShiwanFileUpload.class, params);
                shiwanFileUploadService.saveBatch(listShiwanFileUploads);
                return Result.ok("文件导入成功！数据行数:" + listShiwanFileUploads.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
