package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.toutiao.modules.material.entity.BytedanceEffectVideoInfo;
import cn.com.ctop.toutiao.modules.material.service.IBytedanceEffectVideoInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 有效视频信息
 * @author jeecg-boot
 * @date   2020-07-14
 * @version V1.0
 */
@Slf4j
@Api(tags="有效视频信息")
@RestController
@RequestMapping("/ctop/bytedanceEffectVideoInfo")
public class BytedanceEffectVideoInfoController {
	@Autowired
	private IBytedanceEffectVideoInfoService bytedanceEffectVideoInfoService;

	/**
	  * 分页列表查询
	 * @param bytedanceEffectVideoInfo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "有效视频信息-分页列表查询")
	@ApiOperation(value="有效视频信息-分页列表查询", notes="有效视频信息-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<BytedanceEffectVideoInfo>> queryPageList(BytedanceEffectVideoInfo bytedanceEffectVideoInfo,
																 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
																 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
																 HttpServletRequest req) {
		Result<IPage<BytedanceEffectVideoInfo>> result = new Result<>();
		QueryWrapper<BytedanceEffectVideoInfo> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceEffectVideoInfo, req.getParameterMap());
		queryWrapper.orderByDesc("initial_cost_date");
		Page<BytedanceEffectVideoInfo> page = new Page<>(pageNo, pageSize);
		IPage<BytedanceEffectVideoInfo> pageList = bytedanceEffectVideoInfoService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param bytedanceEffectVideoInfo
	 * @return
	 */
	@AutoLog(value = "有效视频信息-添加")
	@ApiOperation(value="有效视频信息-添加", notes="有效视频信息-添加")
	@PostMapping(value = "/add")
	public Result<BytedanceEffectVideoInfo> add(@RequestBody BytedanceEffectVideoInfo bytedanceEffectVideoInfo) {
		Result<BytedanceEffectVideoInfo> result = new Result<>();
		try {
			bytedanceEffectVideoInfoService.save(bytedanceEffectVideoInfo);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param bytedanceEffectVideoInfo
	 * @return
	 */
	@AutoLog(value = "有效视频信息-编辑")
	@ApiOperation(value="有效视频信息-编辑", notes="有效视频信息-编辑")
	@PutMapping(value = "/edit")
	public Result<BytedanceEffectVideoInfo> edit(@RequestBody BytedanceEffectVideoInfo bytedanceEffectVideoInfo) {
		Result<BytedanceEffectVideoInfo> result = new Result<BytedanceEffectVideoInfo>();
		BytedanceEffectVideoInfo bytedanceEffectVideoInfoEntity = bytedanceEffectVideoInfoService.getById(bytedanceEffectVideoInfo.getId());
		if(bytedanceEffectVideoInfoEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = bytedanceEffectVideoInfoService.updateById(bytedanceEffectVideoInfo);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "有效视频信息-通过id删除")
	@ApiOperation(value="有效视频信息-通过id删除", notes="有效视频信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			bytedanceEffectVideoInfoService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "有效视频信息-批量删除")
	@ApiOperation(value="有效视频信息-批量删除", notes="有效视频信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<BytedanceEffectVideoInfo> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<BytedanceEffectVideoInfo> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.bytedanceEffectVideoInfoService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "有效视频信息-通过id查询")
	@ApiOperation(value="有效视频信息-通过id查询", notes="有效视频信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<BytedanceEffectVideoInfo> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BytedanceEffectVideoInfo> result = new Result<>();
		BytedanceEffectVideoInfo bytedanceEffectVideoInfo = bytedanceEffectVideoInfoService.getById(id);
		if(bytedanceEffectVideoInfo==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(bytedanceEffectVideoInfo);
			result.setSuccess(true);
		}
		return result;
	}
}
