package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.report.entity.BytedancePlanDailyReport;
import cn.com.ctop.toutiao.modules.report.service.IBytedancePlanDailyReportService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * @Description: 广告计划日报表信息
 * @Author: jeecg-boot
 * @Date: 2019-09-11
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "广告计划日报表信息")
@RestController
@RequestMapping("/ctop/bytedancePlanDailyReport")
public class BytedancePlanDailyReportController {
    @Autowired
    private IBytedancePlanDailyReportService bytedancePlanDailyReportService;

    /**
     * 分页列表查询
     *
     * @param bytedancePlanDailyReport
     * @param pageNo
     * @param pageSize
     * @return
     */
    @AutoLog(value = "广告计划日报表信息-分页列表查询")
    @ApiOperation(value = "广告计划日报表信息-分页列表查询", notes = "广告计划日报表信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedancePlanDailyReport>> queryPageList(BytedancePlanDailyReport bytedancePlanDailyReport,
                                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        Result<IPage<BytedancePlanDailyReport>> result = new Result<>();
        QueryWrapper<BytedancePlanDailyReport> queryWrapper = new QueryWrapper<>();
        if (null != bytedancePlanDailyReport.getStatDatetime() && !"".equals(bytedancePlanDailyReport.getStatDatetime())) {
            queryWrapper.eq("stat_datetime", bytedancePlanDailyReport.getStatDatetime().substring(1, 11) + " 00:00:00");
        }
        if (null != bytedancePlanDailyReport.getPlanCode() && !"".equals(bytedancePlanDailyReport.getPlanCode())) {
            queryWrapper.eq("plan_code", bytedancePlanDailyReport.getPlanCode().replace("yk_xxl_jrtt_", ""));
        }
        queryWrapper.orderByDesc("stat_datetime", "order_num");
        Page<BytedancePlanDailyReport> page = new Page<>(pageNo, pageSize);
        IPage<BytedancePlanDailyReport> pageList = bytedancePlanDailyReportService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedancePlanDailyReport
     * @return
     */
    @AutoLog(value = "广告计划日报表信息-添加")
    @ApiOperation(value = "广告计划日报表信息-添加", notes = "广告计划日报表信息-添加")
    @PostMapping(value = "/add")
    public Result<BytedancePlanDailyReport> add(@RequestBody BytedancePlanDailyReport bytedancePlanDailyReport) {
        Result<BytedancePlanDailyReport> result = new Result<BytedancePlanDailyReport>();
        try {
            bytedancePlanDailyReportService.save(bytedancePlanDailyReport);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedancePlanDailyReport
     * @return
     */
    @AutoLog(value = "广告计划日报表信息-编辑")
    @ApiOperation(value = "广告计划日报表信息-编辑", notes = "广告计划日报表信息-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedancePlanDailyReport> edit(@RequestBody BytedancePlanDailyReport bytedancePlanDailyReport) {
        Result<BytedancePlanDailyReport> result = new Result<>();
        BytedancePlanDailyReport bytedancePlanDailyReportEntity = bytedancePlanDailyReportService.getById(bytedancePlanDailyReport.getId());
        if (bytedancePlanDailyReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedancePlanDailyReportService.updateById(bytedancePlanDailyReport);
            if (ok) {
                result.success("修改成功!");
            }else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 编辑
     *
     * @param bytedancePlanDailyReport
     * @return
     */
    @AutoLog(value = "广告计划日报表信息-编辑")
    @ApiOperation(value = "广告计划日报表信息-编辑", notes = "广告计划日报表信息-编辑")
    @PutMapping(value = "/synZybData")
    public Result<BytedancePlanDailyReport> synZybData(@RequestBody BytedancePlanDailyReport bytedancePlanDailyReport) {
        Result<BytedancePlanDailyReport> result = new Result<>();
        BytedancePlanDailyReport bytedancePlanDailyReportEntity = bytedancePlanDailyReportService.getById(bytedancePlanDailyReport.getId());
        if (bytedancePlanDailyReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            bytedancePlanDailyReportEntity.setRegistNum(bytedancePlanDailyReport.getRegistNum());
            bytedancePlanDailyReportEntity.setOrderNum(bytedancePlanDailyReport.getOrderNum());
            boolean ok = bytedancePlanDailyReportService.updateById(bytedancePlanDailyReportEntity);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告计划日报表信息-通过id删除")
    @ApiOperation(value = "广告计划日报表信息-通过id删除", notes = "广告计划日报表信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedancePlanDailyReportService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告计划日报表信息-批量删除")
    @ApiOperation(value = "广告计划日报表信息-批量删除", notes = "广告计划日报表信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedancePlanDailyReport> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedancePlanDailyReport> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedancePlanDailyReportService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告计划日报表信息-通过id查询")
    @ApiOperation(value = "广告计划日报表信息-通过id查询", notes = "广告计划日报表信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedancePlanDailyReport> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedancePlanDailyReport> result = new Result<>();
        BytedancePlanDailyReport bytedancePlanDailyReport = bytedancePlanDailyReportService.getById(id);
        if (bytedancePlanDailyReport == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedancePlanDailyReport);
            result.setSuccess(true);
        }
        return result;
    }
}
