package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.report.entity.BytedanceReportMaterialRetry;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceReportMaterialRetryService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 头条素材报表连接出错数据记录表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-21
 */
@Slf4j
@Api(tags="头条素材报表连接出错数据记录表")
@RestController
@RequestMapping("/bytedance/bytedanceReportMaterialRetry")
public class BytedanceReportMaterialRetryController {
    @Autowired
    private IBytedanceReportMaterialRetryService bytedanceReportMaterialRetryService;

    /**
     * 分页列表查询
     *
     * @param bytedanceReportMaterialRetry
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "头条素材报表连接出错数据记录表-分页列表查询")
    @ApiOperation(value = "头条素材报表连接出错数据记录表-分页列表查询", notes = "头条素材报表连接出错数据记录表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceReportMaterialRetry>> queryPageList(BytedanceReportMaterialRetry bytedanceReportMaterialRetry,
                                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                     HttpServletRequest req) {
        Result<IPage<BytedanceReportMaterialRetry>> result = new Result<>();
        QueryWrapper<BytedanceReportMaterialRetry> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceReportMaterialRetry, req.getParameterMap());
        Page<BytedanceReportMaterialRetry> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceReportMaterialRetry> pageList = bytedanceReportMaterialRetryService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceReportMaterialRetry
     * @return
     */
    @AutoLog(value = "头条素材报表连接出错数据记录表-添加")
    @ApiOperation(value = "头条素材报表连接出错数据记录表-添加", notes = "头条素材报表连接出错数据记录表-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceReportMaterialRetry> add(@RequestBody BytedanceReportMaterialRetry bytedanceReportMaterialRetry) {
        Result<BytedanceReportMaterialRetry> result = new Result<>();
        try {
            bytedanceReportMaterialRetryService.save(bytedanceReportMaterialRetry);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceReportMaterialRetry
     * @return
     */
    @AutoLog(value = "头条素材报表连接出错数据记录表-编辑")
    @ApiOperation(value = "头条素材报表连接出错数据记录表-编辑", notes = "头条素材报表连接出错数据记录表-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceReportMaterialRetry> edit(@RequestBody BytedanceReportMaterialRetry bytedanceReportMaterialRetry) {
        Result<BytedanceReportMaterialRetry> result = new Result<>();
        BytedanceReportMaterialRetry bytedanceReportMaterialRetryEntity = bytedanceReportMaterialRetryService.getById(bytedanceReportMaterialRetry.getId());
        if (bytedanceReportMaterialRetryEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceReportMaterialRetryService.updateById(bytedanceReportMaterialRetry);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "头条素材报表连接出错数据记录表-通过id删除")
    @ApiOperation(value = "头条素材报表连接出错数据记录表-通过id删除", notes = "头条素材报表连接出错数据记录表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceReportMaterialRetryService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "头条素材报表连接出错数据记录表-批量删除")
    @ApiOperation(value = "头条素材报表连接出错数据记录表-批量删除", notes = "头条素材报表连接出错数据记录表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceReportMaterialRetry> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceReportMaterialRetry> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceReportMaterialRetryService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "头条素材报表连接出错数据记录表-通过id查询")
    @ApiOperation(value = "头条素材报表连接出错数据记录表-通过id查询", notes = "头条素材报表连接出错数据记录表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceReportMaterialRetry> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceReportMaterialRetry> result = new Result<>();
        BytedanceReportMaterialRetry bytedanceReportMaterialRetry = bytedanceReportMaterialRetryService.getById(id);
        if (bytedanceReportMaterialRetry == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceReportMaterialRetry);
            result.setSuccess(true);
        }
        return result;
    }
}
