package cn.com.ctop.performanceaccount.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.performanceaccount.entity.PerformanceAccount;
import cn.com.ctop.performanceaccount.service.IPerformanceAccountService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 运营明细表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-05
 */
@Slf4j
@Api(tags = "运营明细表")
@RestController
@RequestMapping("/performanceaccount/performanceAccount")
public class PerformanceAccountController {
    @Autowired
    private IPerformanceAccountService performanceAccountService;

    /**
     * 分页列表查询
     *
     * @param performanceAccount
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "运营明细表-分页列表查询")
    @ApiOperation(value = "运营明细表-分页列表查询", notes = "运营明细表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<PerformanceAccount>> queryPageList(PerformanceAccount performanceAccount,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<PerformanceAccount>> result = new Result<IPage<PerformanceAccount>>();
        QueryWrapper<PerformanceAccount> queryWrapper = QueryGenerator.initQueryWrapper(performanceAccount, req.getParameterMap());
        Page<PerformanceAccount> page = new Page<PerformanceAccount>(pageNo, pageSize);
        IPage<PerformanceAccount> pageList = performanceAccountService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param performanceAccount
     * @return
     */
    @AutoLog(value = "运营明细表-添加")
    @ApiOperation(value = "运营明细表-添加", notes = "运营明细表-添加")
    @PostMapping(value = "/add")
    public Result<PerformanceAccount> add(@RequestBody PerformanceAccount performanceAccount) {
        Result<PerformanceAccount> result = new Result<PerformanceAccount>();
        try {
            performanceAccountService.save(performanceAccount);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param performanceAccount
     * @return
     */
    @AutoLog(value = "运营明细表-编辑")
    @ApiOperation(value = "运营明细表-编辑", notes = "运营明细表-编辑")
    @PutMapping(value = "/edit")
    public Result<PerformanceAccount> edit(@RequestBody PerformanceAccount performanceAccount) {
        Result<PerformanceAccount> result = new Result<PerformanceAccount>();
        PerformanceAccount performanceAccountEntity = performanceAccountService.getById(performanceAccount.getId());
        if (performanceAccountEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = performanceAccountService.updateById(performanceAccount);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "运营明细表-通过id删除")
    @ApiOperation(value = "运营明细表-通过id删除", notes = "运营明细表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            performanceAccountService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "运营明细表-批量删除")
    @ApiOperation(value = "运营明细表-批量删除", notes = "运营明细表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<PerformanceAccount> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<PerformanceAccount> result = new Result<PerformanceAccount>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.performanceAccountService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "运营明细表-通过id查询")
    @ApiOperation(value = "运营明细表-通过id查询", notes = "运营明细表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<PerformanceAccount> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<PerformanceAccount> result = new Result<PerformanceAccount>();
        PerformanceAccount performanceAccount = performanceAccountService.getById(id);
        if (performanceAccount == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(performanceAccount);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<PerformanceAccount> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                PerformanceAccount performanceAccount = JSON.parseObject(deString, PerformanceAccount.class);
                queryWrapper = QueryGenerator.initQueryWrapper(performanceAccount, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<PerformanceAccount> pageList = performanceAccountService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "运营明细表列表");
        mv.addObject(NormalExcelConstants.CLASS, PerformanceAccount.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("运营明细表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<PerformanceAccount> listPerformanceAccounts = ExcelImportUtil.importExcel(file.getInputStream(), PerformanceAccount.class, params);
                performanceAccountService.saveBatch(listPerformanceAccounts);
                return Result.ok("文件导入成功！数据行数:" + listPerformanceAccounts.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
