package cn.com.ctop.performanceappraisal.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.performanceappraisal.entity.Performance;
import cn.com.ctop.performanceappraisal.service.IPerformanceService;
import cn.com.ctop.userefficientvideomap.mapper.UserEfficientVideoMapMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 设计师季度绩效表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-12-10
 */
@Slf4j
@Api(tags = "设计师季度绩效表")
@RestController
@RequestMapping("/ctop/performance")
public class PerformanceController {
    @Autowired
    private IPerformanceService performanceService;
    @Autowired
    private UserEfficientVideoMapMapper userEfficientVideoMapMapper;

    /**
     * 分页列表查询
     *
     * @param performance
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "设计师季度绩效表-分页列表查询")
    @ApiOperation(value = "设计师季度绩效表-分页列表查询", notes = "设计师季度绩效表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<Performance>> queryPageList(Performance performance,
                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                    HttpServletRequest req) {
        Result<IPage<Performance>> result = new Result<IPage<Performance>>();
        QueryWrapper<Performance> queryWrapper = QueryGenerator.initQueryWrapper(performance, req.getParameterMap());
        Page<Performance> page = new Page<Performance>(pageNo, pageSize);
        IPage<Performance> pageList = performanceService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param performance
     * @return
     */
    @AutoLog(value = "设计师季度绩效表-添加")
    @ApiOperation(value = "设计师季度绩效表-添加", notes = "设计师季度绩效表-添加")
    @PostMapping(value = "/add")
    public Result<Performance> add(@RequestBody Performance performance) {
        Result<Performance> result = new Result<Performance>();
        try {
            performanceService.save(performance);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param performance
     * @return
     */
    @AutoLog(value = "设计师季度绩效表-编辑")
    @ApiOperation(value = "设计师季度绩效表-编辑", notes = "设计师季度绩效表-编辑")
    @PutMapping(value = "/edit")
    public Result<Performance> edit(@RequestBody Performance performance) {
        Result<Performance> result = new Result<Performance>();
        Performance performanceEntity = performanceService.getById(performance.getId());
        if (performanceEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = performanceService.updateById(performance);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "设计师季度绩效表-通过id删除")
    @ApiOperation(value = "设计师季度绩效表-通过id删除", notes = "设计师季度绩效表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            performanceService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "设计师季度绩效表-批量删除")
    @ApiOperation(value = "设计师季度绩效表-批量删除", notes = "设计师季度绩效表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<Performance> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<Performance> result = new Result<Performance>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.performanceService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "设计师季度绩效表-通过id查询")
    @ApiOperation(value = "设计师季度绩效表-通过id查询", notes = "设计师季度绩效表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Performance> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<Performance> result = new Result<Performance>();
        Performance performance = performanceService.getById(id);
        if (performance == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(performance);
            result.setSuccess(true);
        }
        return result;
    }

    @GetMapping("/project/list")
    public Map<String, Object> projectList(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize, String appType) {
        return performanceService.getUserOptimizerProjectList(pageNo, pageSize, appType);
    }

    @GetMapping("/account/list")
    public Map<String, Object> accountList(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                           Long projectId, String appType) {
        return performanceService.getUserOptimizerAccountList(appType, projectId, pageNo, pageSize);
    }


    @AutoLog(value = "设计组内设计师季度绩效表-分页列表查询")
    @ApiOperation(value = "设计组内设计师季度绩效表-分页列表查询", notes = "设计组内设计师季度绩效表-分页列表查询")
    @GetMapping(value = "/designerDepartMemberPerformanceList")
    public Result<IPage<Performance>> designerDepartMemberPerformanceList(Performance performance,
                                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                          HttpServletRequest req) {
        Result<IPage<Performance>> result = new Result<IPage<Performance>>();
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //String departId = userEfficientVideoMapMapper.getDepartIdByUserId("e9ca23d68d884d4ebb19d07889727dae");
        String departId = userEfficientVideoMapMapper.getDepartIdByUserId(sysUser.getId());
        List<String> userIdList = userEfficientVideoMapMapper.getUserIdListByDepId(departId);
        ////
        QueryWrapper<Performance> queryWrapper = QueryGenerator.initQueryWrapper(performance, req.getParameterMap());
        queryWrapper.in("user_id", userIdList);

        Page<Performance> page = new Page<Performance>(pageNo, pageSize);
        IPage<Performance> pageList = performanceService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @AutoLog(value = "北京设计师季度绩效列表-分页列表查询")
    @ApiOperation(value = "北京设计师季度绩效列表-分页列表查询", notes = "北京设计师季度绩效列表-分页列表查询")
    @GetMapping(value = "/beijingDesignerPageList")
    public Result<IPage<Performance>> beijingDesignerPageList(Performance performance,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              HttpServletRequest req) {
        Result<IPage<Performance>> result = new Result<IPage<Performance>>();
        QueryWrapper<Performance> queryWrapper = QueryGenerator.initQueryWrapper(performance, req.getParameterMap());
        Page<Performance> page = new Page<Performance>(pageNo, pageSize);
        IPage<Performance> pageList = performanceService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @AutoLog(value = "北京设计师季度绩效列表-分页列表查询")
    @ApiOperation(value = "北京设计师季度绩效列表-分页列表查询", notes = "北京设计师季度绩效列表-分页列表查询")
    @GetMapping(value = "/beijingPlaneDesignerList")
    public Result<IPage<Performance>> beijingPlaneDesignerPageList(Performance performance,
                                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                   HttpServletRequest req) {
        Result<IPage<Performance>> result = new Result<IPage<Performance>>();
        QueryWrapper<Performance> queryWrapper = QueryGenerator.initQueryWrapper(performance, req.getParameterMap());
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //sysUser.setId("b0e4798234df4096936e6bc6d00db3ba");
        queryWrapper.eq("user_id", sysUser.getId());
        Page<Performance> page = new Page<Performance>(pageNo, pageSize);
        IPage<Performance> pageList = performanceService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

}
