package cn.com.ctop.performanceappraisal.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.performanceappraisal.entity.PerformanceOptimizer;
import cn.com.ctop.performanceappraisal.service.IPerformanceOptimizerService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 优化师绩效信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-12-13
 */
@Slf4j
@Api(tags = "优化师绩效信息")
@RestController
@RequestMapping("/ctop/performanceOptimizer")
public class PerformanceOptimizerController {
    @Autowired
    private IPerformanceOptimizerService performanceOptimizerService;

    /**
     * 分页列表查询
     *
     * @param performanceOptimizer
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "优化师绩效信息-分页列表查询")
    @ApiOperation(value = "优化师绩效信息-分页列表查询", notes = "优化师绩效信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<PerformanceOptimizer>> queryPageList(PerformanceOptimizer performanceOptimizer,
                                                             @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                             HttpServletRequest req) {
        Result<IPage<PerformanceOptimizer>> result = new Result<>();
        QueryWrapper<PerformanceOptimizer> queryWrapper = QueryGenerator.initQueryWrapper(performanceOptimizer, req.getParameterMap());
        Page<PerformanceOptimizer> page = new Page<PerformanceOptimizer>(pageNo, pageSize);
        IPage<PerformanceOptimizer> pageList = performanceOptimizerService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param performanceOptimizer
     * @return
     */
    @AutoLog(value = "优化师绩效信息-添加")
    @ApiOperation(value = "优化师绩效信息-添加", notes = "优化师绩效信息-添加")
    @PostMapping(value = "/add")
    public Result<PerformanceOptimizer> add(@RequestBody PerformanceOptimizer performanceOptimizer) {
        Result<PerformanceOptimizer> result = new Result<PerformanceOptimizer>();
        try {
            performanceOptimizerService.save(performanceOptimizer);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param performanceOptimizer
     * @return
     */
    @AutoLog(value = "优化师绩效信息-编辑")
    @ApiOperation(value = "优化师绩效信息-编辑", notes = "优化师绩效信息-编辑")
    @PutMapping(value = "/edit")
    public Result<PerformanceOptimizer> edit(@RequestBody PerformanceOptimizer performanceOptimizer) {
        Result<PerformanceOptimizer> result = new Result<PerformanceOptimizer>();
        PerformanceOptimizer performanceOptimizerEntity = performanceOptimizerService.getById(performanceOptimizer.getId());
        if (performanceOptimizerEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = performanceOptimizerService.updateById(performanceOptimizer);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "优化师绩效信息-通过id删除")
    @ApiOperation(value = "优化师绩效信息-通过id删除", notes = "优化师绩效信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            performanceOptimizerService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "优化师绩效信息-批量删除")
    @ApiOperation(value = "优化师绩效信息-批量删除", notes = "优化师绩效信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<PerformanceOptimizer> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<PerformanceOptimizer> result = new Result<PerformanceOptimizer>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.performanceOptimizerService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "优化师绩效信息-通过id查询")
    @ApiOperation(value = "优化师绩效信息-通过id查询", notes = "优化师绩效信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<PerformanceOptimizer> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<PerformanceOptimizer> result = new Result<PerformanceOptimizer>();
        PerformanceOptimizer performanceOptimizer = performanceOptimizerService.getById(id);
        if (performanceOptimizer == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(performanceOptimizer);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<PerformanceOptimizer> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                PerformanceOptimizer performanceOptimizer = JSON.parseObject(deString, PerformanceOptimizer.class);
                queryWrapper = QueryGenerator.initQueryWrapper(performanceOptimizer, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<PerformanceOptimizer> pageList = performanceOptimizerService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "优化师绩效信息列表");
        mv.addObject(NormalExcelConstants.CLASS, PerformanceOptimizer.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("优化师绩效信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<PerformanceOptimizer> listPerformanceOptimizers = ExcelImportUtil.importExcel(file.getInputStream(), PerformanceOptimizer.class, params);
                performanceOptimizerService.saveBatch(listPerformanceOptimizers);
                return Result.ok("文件导入成功！数据行数:" + listPerformanceOptimizers.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /**
     * 优化师绩效详情
     *
     * @param pageNo
     * @param pageSize
     * @param mediaType
     * @return
     */
    @GetMapping(value = "/detail")
    @ResponseBody
    public Map<String, Object> getPerformanceDetail(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                    @RequestParam(name = "mediaType", defaultValue = "2") String mediaType) {
        return performanceOptimizerService.getPerformanceDetail(mediaType, pageNo, pageSize);
    }

    /**
     * 优化师绩效排行榜
     *
     * @param mediaType
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/ranking/list")
    @ResponseBody
    public Map<String, Object> getRankingList(@RequestParam(name = "mediaType", defaultValue = "2") String mediaType,
                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                              @RequestParam(name = "pageSize", defaultValue = "3") Integer pageSize,
                                              @RequestParam(name = "orgCode", defaultValue = "A01") String orgCode) {
        return performanceOptimizerService.getRankingList(mediaType, pageNo, pageSize, orgCode);
    }

    /**
     * 获取项目经理相关数据
     *
     * @param
     * @param mediaType
     * @return
     */
    @GetMapping(value = "/manager/detail")
    @ResponseBody
    public Map<String, Object> getManagerPerformanceDetail(@RequestParam(name = "orgCode", defaultValue = "A01") String orgCode,
                                                           @RequestParam(name = "mediaType", defaultValue = "2") String mediaType) {
        return performanceOptimizerService.getManagerPerformanceDetail(mediaType, orgCode);
    }


    /**
     * 计算运营绩效
     */
    @GetMapping(value = "/yunyingPerformance")
    @ResponseBody
    public void yunyingPerformance(@RequestParam(name = "year") Integer year,
                                   @RequestParam(name = "quarter") Integer quarter) {

        Map<String, String> map = DateUtils.quarterStartEndDate(year, quarter);
        //performanceOptimizerService.toutiaoYunyingQuarterPerformance(year, quarter, map.get("startDate"), map.get("endDate"));
        //performanceOptimizerService.kuaishouYunyingQuarterPerformance(year, quarter, map.get("startDate"), map.get("endDate"));

        performanceOptimizerService.toutiaoYunyingQuarterPerformance(year, quarter, "2020-01-01", "2020-03-31");
        performanceOptimizerService.kuaishouYunyingQuarterPerformance(year, quarter, "2020-01-01", "2020-03-31");

    }

    /**
     * 计算运营绩效
     */
    @GetMapping(value = "/kuaishouYunyingLeaderQuarterPerformance")
    @ResponseBody
    public void kuaishouYunyingLeaderQuarterPerformance(@RequestParam(name = "year") Integer year,
                                                        @RequestParam(name = "quarter") Integer quarter) {

        Map<String, String> map = DateUtils.quarterStartEndDate(year, quarter);
        performanceOptimizerService.kuaishouYunyingLeaderQuarterPerformance(year, quarter);
    }

}
